
#include "genedrop.h"

main(int argc, char *argv[])
{
    atexit(print_tail);
    print_head(PNAM);


    if (!proc_pars(argc, argv, PNAM))	/* process input parameter 	*/
	exit(EXIT_FAILURE);	 	/* statements			*/	

    if (!proc_drop())	                /* check statements for  	*/
	exit(EXIT_FAILURE);		/* completeness and consistency */
                                        /* and prepare controls for     */
                                        /* genedrop			*/	

    if (!proc_fils())	                /* set up pedigree file   	*/
	exit(EXIT_FAILURE);		/* parameters                   */
  
    if (!chk_pedfil())                  /* reads pedigree file          */
        give_up(ERR_qit);               /* checks pedigree for errors   */
   
    sortChron();			/* sorts key table on chron no.	*/

    allocGenes();			/* allocates arrays 		*/
 
    initSeeds1();                       /* sets seeds for markers       */

    dropMarkerGenes();             	/* assigns genes to markers 	*/

    initSeeds2();         		/* saves marker seeds, 		*/
					/* sets trait seeds		*/

    dropTraitGenes();	        	/* assigns genes to linkd traits*/
                                        /* then to unlinkd traits       */

    initSeeds3();			/* saves trait seeds		*/

    if (!ChronOut)			/* if output order is same as 	*/
        sortIx();			/* input order, resorts		*/	

    openSimPed();			/* opens the output file        */

    writSimPed();		        /* writes out the pedigree file */
    fclose(oPedF);

   if (MarkSeedsOut  || TraitSeedsOut)
   {
        openSeedOut();
        writSeeds();     		/* writes out the seeds		*/
        fclose(oSeedF);
   }

    freeStuff();			/* frees data structures    	*/

    exit(EXIT_SUCCESS);
}

/*. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */ 

void allocGenes(void)
{
    pGenoDat = malloc(nInd * NLoci * sizeof(GENO));
    alloc_chk(WHERE, pGenoDat);

    if (NTraits)
    {
        pPhenoDat = malloc(nInd * NTraits * sizeof(PHENO));
        alloc_chk(WHERE, pPhenoDat);
    }

    pKDat = malloc(nInd * NLoci * sizeof(KDAT));
    alloc_chk(WHERE, pKDat);

    return;
}

void initSeeds1(void)
{
    setsd(MarkSeedC, MarkSeedT);

    return;
}

void initSeeds2(void)
{
    getsd(&MarkSeedC, &MarkSeedT);

    if (!TraitSeedSet)
    {
        TraitSeedC = MarkSeedC;
        TraitSeedT = MarkSeedT;

         if (NTraits )
        {
            fprintf(stdout," Hexadecimal trait seeds:   %#x %#x\n\n", 
                        TraitSeedC, TraitSeedT);
            fflush(stdout);
        }
    }

    setsd(TraitSeedC, TraitSeedT);

    return;
}

void initSeeds3(void)
{
    getsd(&TraitSeedC, &TraitSeedT);

    return;
}

void dropMarkerGenes(void)
{
    KEYS_PP ppK;
    KDAT_PTR pKD;
    GENO_PTR pG;
    RMAP_PTR pR;
    ALLELE_PTR pA;
    int iLoc;

    if (NMarks)                         /* Markers first ...		*/
    {
   					/* for each individual		*/
        for (ppK = ppKeyTab; ppK < ppKeyTab + nInd; ++ppK)	
        {
            pKD = pKDat + ((*ppK)->ix - 1) * NLoci;
            pG = pGenoDat + ((*ppK)->ix -1 ) * NLoci;			
            pR = PRecombMap;
            pA = PAlleleFreq;
            iLoc = 0;
        			
            while (iLoc <  NLoci) 		/* for each locus	*/
            {
                if (pR->recombTyp == FM || pR->recombTyp == OM)
              
						/* assigns marker genes	*/	
                    if ((*ppK)->founder)    	/* for founders		*/ 
                        foundGenes(pA, pKD, pG, iLoc);   
                    else                    	/* and descendants	*/
                        inheritGenes(ppK, pR, pKD, pG, iLoc); 
                     
                ++pR;
                ++pA;
                ++iLoc;
                ++pG;
            }
        }
    }
    return;
}

void dropTraitGenes(void)
{
    KEYS_PP ppK;
    KDAT_PTR pKD;
    GENO_PTR pG;
    PHENO_PTR pPh;
    RMAP_PTR pR;
    ALLELE_PTR pA;
    TPARS_PTR pTr;
    int iLoc, iTrait;

    if (NTraits - NUnlnkd)              /* Now traits ...    		*/
    {
   					/* for each individual		*/
        for (ppK = ppKeyTab; ppK < ppKeyTab + nInd; ++ppK)	
        {
            pKD = pKDat + ((*ppK)->ix - 1) * NLoci;
            pG = pGenoDat + ((*ppK)->ix - 1) * NLoci;			
            pPh = pPhenoDat + ((*ppK)->ix - 1) * NTraits;	
            pR = PRecombMap;
            pA = PAlleleFreq;
            pTr = PTraitPars;
            iLoc = 0;
            iTrait = 0;
        				
            while (iLoc <  NLoci) 		/* for each locus	*/
            {
                if (pR->recombTyp == FT || pR->recombTyp == IT
                                        || pR->recombTyp == ET) 
                {
						/* assigns linked trait */
                    if ((*ppK)->founder)  	/* genes for founders	*/ 
                    {
                        foundGenes(pA, pKD, pG, iLoc);   
                        foundTrait(pPh, pTr);
                    }
                    else                    	/* and descendents	*/
                    {
                        inheritGenes(ppK, pR, pKD, pG, iLoc); 
                        inheritTrait(ppK, pPh, pTr, iTrait);
                    }
                    makQuant(pG, pPh, pTr);
                }         
                if (pR->recombTyp == FT || pR->recombTyp == IT
                                        || pR->recombTyp == ET
                                        || pR->recombTyp == UT) 
                {
                    ++pPh;
                    ++pTr;
                    ++iTrait;
                }
                ++pR;
                ++pA;
                ++iLoc;
                ++pG;
            }
        }
    }
    if (NUnlnkd)                        /* And unlinked traits ...	*/
    {
   					/* for each individual		*/
        for (ppK = ppKeyTab; ppK < ppKeyTab + nInd; ++ppK)	
        {
            pKD = pKDat + ((*ppK)->ix - 1) * NLoci;
            pG = pGenoDat + ((*ppK)->ix - 1) * NLoci; 
            pPh = pPhenoDat + ((*ppK)->ix - 1) * NTraits; 
            pR = PRecombMap;
            pA = PAlleleFreq;
            pTr = PTraitPars;
            iLoc = 0;
            iTrait = 0;
        				
            while (iLoc <  NLoci) 		/* for each locus	*/
            {
                if (pR->recombTyp == UT)
                {
						/* assigns unlinked     */
                    if ((*ppK)->founder) 	/* trait genes for      */
						/* founders		*/ 
                    {
                        foundGenes(pA, pKD, pG, iLoc);   
                        foundTrait(pPh, pTr);
                    }
                    else                    	/* and descendents	*/
                    {
                        inheritGenes(ppK, pR, pKD, pG, iLoc); 
                        inheritTrait(ppK, pPh, pTr, iTrait);
                    }
                    makQuant(pG, pPh, pTr);
                }         
                if (pR->recombTyp == FT || pR->recombTyp == IT
                                        || pR->recombTyp == ET
                                        || pR->recombTyp == UT) 
                {
                    ++pPh;
                    ++pTr;
                    ++iTrait;
                }
                ++pR;
                ++pA;
                ++iLoc;
                ++pG;
            }
        }
    }
}

void openSimPed(void)
{
    if (!(oPedF = open_fil(OPED)))
        give_up(ERR_qit);

    return;
}

void writSimPed(void)
{
    KEYS_PP ppK = ppKeyTab;
    PED_PTR pP;
    PEDI_PTR pPI;
    GENO_PTR pG;
    PHENO_PTR pPh;
    RMAP_PTR pR = PRecombMap;
    TOUT_PTR pT = PTraitOut;
    
    int i, j, ixInd;

    widPNam = intWid(0, maxPNam);       /*get no. chars for writing     */
    for (j = 0; j < NNInts; ++j)
        widPInt[j] = intWid(minPInt[j], maxPInt[j]);

					/*for each individual:		*/
    for (i = 0; i < nInd; ++i)
    {
        ixInd = (*ppK)->ix - 1;		/*gets index for next individual*/

        pP = pPedDat + ixInd;           /*outputs standard ints (name,  */
				        /*parents, gender, and if       */
                                        /*required, observed indicator) */	
        for (j = 0; j < NStdOut; ++j)
        {
            switch (StdOut[j]) 
            { 
                case NAM: 
                {
                    fprintf(oPedF, "%*d", widPNam, pP->nam);
                    break; 
                }   
                case PA: 
                {
                    fprintf(oPedF, " %*d", widPNam, pP->namPa);
                    break; 
                }   
                case MA:
                {
                    fprintf(oPedF, " %*d", widPNam, pP->namMa);
                    break;
                }   
                case SEX:
                {
                    fprintf(oPedF, " %2d", pP->sex);
                    break;
                } 
                case OBS:
                {
                    fprintf(oPedF, " %d", pP->obsrvd);
                    break;
                } 
                default:
                    break;
            }
        }                           	/*outputs integers          	*/
        pPI = pIntDat + NNInts * ixInd;	 

        for (j = 0; j < NNInts; ++j)
            fprintf(oPedF, " %*d", widPInt[j], *(pPI + j)); 

        pG = pGenoDat + NLoci * ixInd;  /*outputs marker genes  	*/
        pR = PRecombMap;

        for (j = 0; j < NLoci; ++j)
        {
            if (pR->recombTyp == FM || pR->recombTyp == OM)
                if (!AllVarOut && !pP->obsrvd)
                    fprintf(oPedF, "  %*d %*d", WidAlle, MISSGENE, 
                                                WidAlle, MISSGENE); 
                else if (pG->genoPa < pG->genoMa)
                    fprintf(oPedF, "  %*d %*d", 
                            WidAlle, pG->genoPa + 1, WidAlle, pG->genoMa + 1);
                else
                    fprintf(oPedF, "  %*d %*d", 
                            WidAlle, pG->genoMa + 1, WidAlle, pG->genoPa + 1);
            ++pG;
            ++pR;
        }
                       			/*now puts out traits		*/
        pT = PTraitOut;

        if (NTraits > 0)
            fprintf(oPedF, " ");

        for (j = 0; j < NTraits; ++j)
        {
            pPh = pPhenoDat + NTraits * ixInd + pT->ixPh;

            if (AllVarOut)		/*optionally outputs trait genes*/
            {
                pG = pGenoDat + NLoci * ixInd + pT->ixG;
                if (pG->genoPa < pG->genoMa)
                    fprintf(oPedF, "  %d %d", 
                                 pG->genoPa + 1, pG->genoMa + 1);
                else
                    fprintf(oPedF, "  %d %d", 
                                 pG->genoMa + 1, pG->genoPa + 1);
            }
                          		/*outputs quantitative data	*/

            if (AllVarOut)		/*optionally outputs variances	*/
            {
                fprintf(oPedF, " %*.*f %*.*f %*.*f", 
                                 pT->wid, pT->dec, pPh->addi,
                                 pT->wid, pT->dec, pPh->resi,
                                 pT->wid, pT->dec, pPh->quant);
            }	
            else
                if (!pP->obsrvd)
                    fprintf(oPedF, " %*d", pT->wid, MISSING);
                else
                    fprintf(oPedF, " %*.*f", pT->wid, pT->dec, pPh->quant);
            ++pT;
        }
 			
	fprintf(oPedF, "\n");		/*ends current record		*/
        fflush(oPedF);
        
        ++ppK;
    }
    return;
}

void openSeedOut(void)
{
    if (!(oSeedF = open_fil(OSEED)))
        give_up(ERR_qit);

    return;
}
void writSeeds(void)
{
    if (MarkSeedsOut)
        fprintf(oSeedF, " set marker seeds %#x %#x\n", MarkSeedC, MarkSeedT);
    if (TraitSeedsOut)
        fprintf(oSeedF, " set trait seeds  %#x %#x\n", TraitSeedC, TraitSeedT);
    
    return;
}

void freeStuff(void)
{
    free(pPedDat);
    pPedDat = NULL;
    if (NNInts)
    {
        free(pIntDat);
        pIntDat = NULL;
    }
    free(pFirstKeyTab);
    pFirstKeyTab = NULL;
    free(ppKeyTab);
    ppKeyTab = NULL;
    free(pGenoDat);
    pGenoDat = NULL;
    if (NTraits)
    {
        free(pPhenoDat);
        pPhenoDat = NULL;
    }
    free(pKDat);
    pKDat = NULL;
   
    return;
}

void foundGenes(ALLELE_PTR pA, KDAT_PTR pKD, GENO_PTR pG, int iLoc)
					/*assigns alleles to founders	*/
{				
    int j;
    double rM, rP;

    rM = rans();
    rP = rans();

    for (j = 0; j < pA->nAlleles && rM > *((pA->pCumFreq)+j); ++j)
                ;	
    pG->genoMa = j;			/* selects maternal allele	*/

    for (j = 0; j < pA->nAlleles && rP > *((pA->pCumFreq)+j); ++j)
                ;
    pG->genoPa = j;			/* selects paternal allele	*/

    (pKD + iLoc)->kMat = FOUNDER;
    (pKD + iLoc)->kPat = FOUNDER;

    return;
}

void inheritGenes(KEYS_PP ppK, RMAP_PTR pR, KDAT_PTR pKD, 
                  GENO_PTR pG, int iLoc)
					/*assigns k's and thus alleles	*/
{					/* to nonfounders		*/
    int kM, kP, kA, kB;
    double rM, rP;
    
    rM = rans();
    rP = rans();

    if (pR->recombTyp == FM || pR->recombTyp == UT)
    {					/* first marker on chromosome	*/
					/* or unlinked trait		*/
        if (rM < .5)
            kM = GM;
        else
            kM = GF;

        if (rP < .5)
            kP = GM;
        else
            kP = GF;
    }

    else if (pR->recombTyp == OM || pR->recombTyp == ET)
    {					/* is not first marker on chrom,*/
					/* or, is trait with preceding  */
					/* locus and no following marker*/
        kM = (pKD + pR->iA)->kMat;
        if (rM < pR->probA)
            kM = !kM;

        kP = (pKD + pR->iA)->kPat;
        if (rP < pR->probA)
            kP = !kP;
    }
			
    else if (pR->recombTyp == FT)
					/* trait is first locus on chrom*/
    {
        kM = (pKD + pR->iB)->kMat;
        if (rM < pR->probB)
            kM = !kM;

        kP = (pKD + pR->iB)->kPat;
        if (rP < pR->probB)
            kP = !kP;
    }

    else if (pR->recombTyp == IT)
					/* trait has preceding locus 	*/
    {					/* and following marker		*/
        kA = (pKD + pR->iA)->kMat;
        kB = (pKD + pR->iB)->kMat;
        if (kA == kB)
        {
            if (rM < pR->probA)
                kM = !kA;
            else
                kM = kA;
        }
        else
        {
            if (rM < pR->probB)
                kM = kA;
            else
                kM = kB;
        }

        kA = (pKD + pR->iA)->kPat;
        kB = (pKD + pR->iB)->kPat;
        if (kA == kB)
        {
            if (rP < pR->probA)
                kP = !kA;
            else
                kP = kA;
        }
        else
        {
            if (rP < pR->probB)
                kP = kA;
            else
                kP = kB;
        }
    }

    (pKD + iLoc)->kMat = kM;

    if (kM == GM)
        pG->genoMa = (pGenoDat + ((*ppK)->ixMa - 1) * NLoci + iLoc)->genoMa;
    else if (kM == GF)
        pG->genoMa = (pGenoDat + ((*ppK)->ixMa - 1) * NLoci + iLoc)->genoPa;
    else
        give_up_where(WHERE, ERR_inh, "inheritGenes", (*ppK)->nam);

    (pKD + iLoc)->kPat = kP;

    if (kP == GM)
        pG->genoPa = (pGenoDat + ((*ppK)->ixPa - 1) * NLoci + iLoc)->genoMa;
    else if (kP == GF)
        pG->genoPa = (pGenoDat + ((*ppK)->ixPa - 1) * NLoci + iLoc)->genoPa;
    else
        give_up_where(WHERE, ERR_inh, "inheritGenes", (*ppK)->nam);

    return;
}
void foundTrait(PHENO_PTR pPh, TPARS_PTR pTP)
{
    pPh->addi = normal(0., pTP->addi_term_found);

    return;
}
void inheritTrait(KEYS_PP ppK, PHENO_PTR pPh, TPARS_PTR pTP, int iT)
{
    double addiMa, addiPa;

    addiMa = (pPhenoDat + ((*ppK)->ixMa - 1) * NTraits + iT)->addi;
    addiPa = (pPhenoDat + ((*ppK)->ixPa - 1) * NTraits + iT)->addi;
    pPh->addi = normal((addiMa + addiPa)/2., pTP->addi_term);

    return;
}
void makQuant(GENO_PTR pG, PHENO_PTR pPh, TPARS_PTR pTP)
{
    int i;

    pPh->resi = normal(0., pTP->resi_term);
    i = pG->genoMa + pG->genoPa;		/* 0 for 0,0;		*/
						/* 1 for 0,1 or 1,0	*/
						/* 2 for 1,1		*/
    pPh->quant = pTP->means[i] + pPh->addi + pPh->resi;

    return;
}

