
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*      errspars.h:  error messages produced by Pars/ functions        */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef ERRSPARS_H		
#define ERRSPARS_H

#define E 'E'
#define W 'W'
#define F 'F'
#define Q 'Q'

static int      n_warn;
static int      n_err;

/* err_typ:  error codes which result in the error messages and error	*/
/*           operations defined in errors[] below			*/

static enum  
    { ERR_NUL,
      ERR_FNU,
      ERR_PCL,
      ERR_FCL,
      ERR_EPF,
      ERR_EIF,
      ERR_OPF,
      ERR_SIF,
      ERR_CLF,
      ERR_KEY,
      ERR_DAT,
      ERR_HEX,
      ERR_STR,
      ERR_XTR,
      ERR_NOT,
      ERR_RED,
      ERR_PED,
      ERR_PEX,
      ERR_ALU,
      ERR_ALP,
      ERR_ALC,
      ERR_ALO,
      ERR_ALN,
      ERR_PF1,
      ERR_PF2,
      ERR_GND,
      ERR_OPA,
      ERR_ASG,
      ERR_IGG,
      ERR_ORD,
      ERR_TRO,
      ERR_TRR,
      ERR_FFO,
      ERR_FFD,
      ERR_MNR,
      ERR_FFR,
      ERR_LVM,
      ERR_VAM,
      ERR_COM,
      ERR_CFM,
      ERR_CFO,
      ERD_CHO,
      ERD_MK0,
      ERD_MKM,
      ERD_TR0,
      ERD_TRM,
      ERD_RSM,
      ERD_RMP,
      ERD_RCS,
      ERD_UTM,
      ERD_MDM,
      ERD_MRM,
      ERD_MPM,
      ERD_TMM,
      ERD_MDO,
      ERD_MFO,
      ERD_MPO,
      ERD_MPD,
      ERD_TDO,
      ERD_TFO,
      ERD_MKO,
      ERD_TRO,
      ERD_MLO,
      ERD_TLO,
      ERD_AFO,
      ERD_SMA,
      ERD_MAF,
      ERD_TAF,
      ERD_TGM,
      ERD_TGR,
      ERD_AVR,
      ERD_RVR,
      ERD_VMN,
      ERR_EIM,
      ERF_PE0,
      ERF_PE1,
      ERF_PE2,
      ERF_IPR,
      ERF_NIN,
      ERF_NRL,
      ERR_STN,
      ERR_SFN,
      ERR_TRF,
      ERR_FFF,
      ERR_SEL,
      ERR_VA0,
      ERR_VR0,
      ERR_VSA,
      ERR_VSR,
      ERR_VAN,
      ERR_VRN,
      ERR_CSA,
      ERR_CSR,
      ERR_CFA,
      ERR_CFR,
      ERR_CAF,
      ERR_CRF,
      ERR_CAZ,
      ERR_CRZ,
      ERR_PDA,
      ERR_PDR,
      ERR_ET1,
      ERR_ET2,
      ERR_PT1,
      ERD_SIM,
      ERD_CHR,
      ERD_NMM,
      ERD_NOM,
      ERD_NTM,
      ERD_NOT,
      ERD_MMX,
      ERD_LLL,
      ERD_LUL,
      ERD_NML,
      ERD_TAV,
      ERD_TRV,
      ERD_NAR,
      ERD_SVR,
      ERD_NTL,
      ERD_NGM,
      ERD_MTD,
      ERD_SEE,
      ERD_OTS,
      ERD_OMS,
      ERD_SSD,
      ERD_OPF,
      ERD_OST,
      ERD_OSM,
      ERR_FFL,
      ERR_FFN,
      ERR_CHG,
      ERR_EIP,
      ERR_OEF,
      ERR_EIE,
      ERR_OBS,
      ERR_MAX,
      ERR_FAT,
      ERR_QIT 
      } err_typ ;

static char errnumb[80];
static char * err_txt;		   /*ptr to text for txtflg below	*/

/* error message table							*/

/* error level is   'F' causes immediate termination of program		*/
/*		    'E' causes termination of program after all input	*/
/*			statements are processed			*/
/*		    'W' warning statement only				*/

static struct _errors {
    char * msg;				/*text for error		*/	
    char   level;			/*error level			*/
    int    tokflg;			/*whether tokens are to be prntd*/
    int    txtflg;			/*whether an arg is to be prntd	*/ 
    int    statflg;			/*whether correct form printed	*/
               } *perr;

static struct _errors errors[] = {
{"***"							        ,F,N,N,N}, /*ERR_NUL*/
{" File type \"%s\" from command line not recognized"           ,E,N,Y,N}, /*ERR_FNU*/
{" File type \"%s\" from command line not used by this program" ,W,N,Y,N}, /*ERR_PCL*/
{" Incorrect specification of files in command line.  Correct form is:\n\n"
 "   %s"                                                        ,E,N,Y,N}, /*ERR_FCL*/
{" No pedigree file specified in command line or \"input pedigree file\" statement" 
                                                                ,E,N,N,N}, /*ERR_EPF*/
{" No parameter file specified in command line"	                ,F,N,N,N}, /*ERR_EIF*/
{" Can't open %s"			          	        ,E,N,Y,N}, /*ERR_OPF*/
{" No parameter statements in file \"%s\""                      ,W,N,Y,N}, /*ERR_SIF*/
{" Error in closing file"				        ,W,N,N,N}, /*ERR_CLF*/
{" Unrecognizable word or word order"			        ,E,Y,N,N}, /*ERR_KEY*/
{" Missing data in parameter statement"	         	        ,E,Y,N,Y}, /*ERR_DAT*/
{" Format error in hexadecimal data value"                      ,E,Y,N,Y}, /*ERR_HEX*/
{" Expected string in statement"			        ,E,Y,N,Y}, /*ERR_STR*/
{" Too many parameters in input statement"		        ,E,Y,N,Y}, /*ERR_XTR*/
{" This statement is not implemented for program \"%s\""        ,W,Y,Y,N}, /*ERR_NOT*/
{" Overrides previous statement of same type"		        ,W,Y,N,N}, /*ERR_RED*/
{" Pedigree size must be greater than 2"		        ,E,Y,N,N}, /*ERR_PED*/
{" Pedigree size is greater than default maximum size of %s"    ,W,Y,Y,N}, /*ERR_PEX*/
{" Overrides default maximum pedigree size of %s\n"
 "    Program \"unibig\" is recommended instead for large pedigrees"
                                                                ,W,Y,Y,N}, /*ERR_ALU*/
{" Overrides default maximum pedigree size of %s"  	        ,W,Y,Y,N}, /*ERR_ALP*/
{" Overrides default maximum component size of %s"              ,W,Y,Y,N}, /*ERR_ALC*/
{" Overrides default maximum number of observed individuals of %s"        
                                                                ,W,Y,Y,N}, /*ERR_ALO*/
{" Overrides default maximum name of %s"	  	        ,W,Y,Y,N}, /*ERR_ALN*/
{" Number of integers out of range"			        ,E,Y,N,N}, /*ERR_PF1*/
{" Number of reals is out of range"			        ,E,Y,N,N}, /*ERR_PF2*/
{" Overrides previous gender present/absent statement"	        ,W,Y,N,N}, /*ERR_GND*/
{" Overrides previous observed present/absent statement"        ,W,Y,N,N}, /*ERR_OPA*/
{" Overrides previous ignore gender statement"	                ,W,Y,N,N}, /*ERR_ASG*/
{" Overrides previous assign gender statement"		        ,W,Y,N,N}, /*ERR_IGG*/
{" Overrides previous record order statement"		        ,W,Y,N,N}, /*ERR_ORD*/
{" Trait number is out of range"			        ,E,Y,N,N}, /*ERR_TRO*/
{" Overrides previous statement for same trait"		        ,W,Y,N,N}, /*ERR_TRR*/
{" Fixed effect number is out of range"		              	,E,Y,N,N}, /*ERR_FFO*/
{" Repeated fixed effect number in statement"		        ,W,Y,N,N}, /*ERR_FFD*/
{" Overrides previous mean for same trait"	                ,W,Y,N,N}, /*ERR_MNR*/
{" Overrides previous fixed effect levels for same trait"       ,W,Y,N,N}, /*ERR_FFR*/
{" Too many starting values for fixed effect levels"	        ,E,Y,N,N}, /*ERR_LVM*/
{" Too many starting values for variances"		        ,E,Y,N,N}, /*ERR_VAM*/
{" Too many starting values for covariances or correlations"    ,E,Y,N,N}, /*ERR_COM*/
{" Too many fit flags for covariances or correlations"	        ,E,Y,N,N}, /*ERR_CFM*/
{" Statement ignored.  %s specified in command line"            ,W,Y,Y,N}, /*ERD_CFO*/
{" Chromosome number is out of range"                           ,E,Y,N,N}, /*ERD_CHO*/
{" At least one marker expected"                                ,E,Y,N,N}, /*ERD_MK0*/
{" Too many markers"                                            ,E,Y,N,N}, /*ERD_MKM*/
{" Bad number of traits"                                        ,E,Y,N,N}, /*ERD_TR0*/
{" Too many traits"                                             ,E,Y,N,N}, /*ERD_TRM*/
{" Overrides previous simulation statement"                     ,W,Y,N,N}, /*ERD_RSM*/
{" Overrides previous map statement"                            ,W,Y,N,N}, /*ERD_RMP*/
{" Overrides previous statement for this chromosome"            ,W,Y,N,N}, /*ERD_RCS*/
{" Too many unlinked traits"                                    ,E,Y,N,N}, /*ERD_UTM*/
{" Too many marker distances"                                   ,E,Y,N,N}, /*ERD_MDM*/
{" Too many marker recombination fractions"                     ,E,Y,N,N}, /*ERD_MRM*/
{" Too many marker positions"                                   ,E,Y,N,N}, /*ERD_MPM*/
{" There must be the same number of traits, markers and\n"
 "        distances, positions or recombination fractions"      ,E,Y,N,N}, /*ERD_TMM*/
{" Marker distances must be >= 0. and <= 500. cM"               ,E,Y,N,N}, /*ERD_MDO*/
{" Marker recombination fractions must be >= 0. and < .5"       ,E,Y,N,N}, /*ERD_MFO*/
{" Marker positions must be > -500. and <= 500. cM"             ,E,Y,N,N}, /*ERD_MPO*/
{" Marker positions must be in increasing order"                ,E,Y,N,N}, /*ERD_MPD*/
{" Trait distances must be >= 0. and <= 500. cM"                ,E,Y,N,N}, /*ERD_TDO*/
{" Trait recombination fractions must be >= 0. and < .5"        ,E,Y,N,N}, /*ERD_TFO*/
{" Marker number out of range"                                  ,E,Y,N,N}, /*ERD_MKO*/
{" Trait number out of range"                                   ,E,Y,N,N}, /*ERD_TRO*/
{" Too few or too many marker alleles"                          ,E,Y,N,N}, /*ERD_MLO*/
{" Expected two alleles for trait locus"                        ,E,Y,N,N}, /*ERD_TLO*/
{" Allele frequencies must be between 0. and 1."                ,E,Y,N,N}, /*ERD_AFO*/
{" Sum of allele frequencies is not 1.0:  frequencies will be normalized"
                                                                ,W,Y,N,N}, /*ERD_SMA*/
{" Overrides previous allele frequencies for marker(s)"         ,W,Y,N,N}, /*ERD_MAF*/
{" Overrides previous allele frequencies for trait(s)"          ,W,Y,N,N}, /*ERD_TAF*/
{" There must be 3 mean values, one each for the (1 1), the (1 2) or (2 1),\n"
 "           and the (2 2) genotypes"                           ,E,Y,N,N}, /*ERD_TGM*/
{" Overrides previous genotype means for trait(s)"              ,W,Y,N,N}, /*ERD_TGR*/
{" Overrides previous additive variance(s) for trait(s)"        ,W,Y,N,N}, /*ERD_AVR*/
{" Overrides previous residual variance(s) for trait(s)"        ,W,Y,N,N}, /*ERD_RVR*/
{" Negative variance"                                           ,E,Y,N,N}, /*ERD_VMN*/
{" Number of eigenvalues greater than pedigree size"	        ,E,N,N,N}, /*ERR_EIM*/
{" \"input pedigree size\" statement missing"             ,E,N,N,N}, /*ERF_PE0*/
{" Pedigree size is too large.  Use \"unibig\" instead for big pedigrees"  
					                  ,E,N,N,N}, /*ERF_PE1*/
{" Pedigree size too large.  Use \"allow pedigree size\" statement to override default"
					                  ,E,N,N,N}, /*ERF_PE2*/
{" \"input pedigree record integers ...\" statement missing"
                                                         ,E,N,N,N}, /*ERF_IPR*/
{" Expected \"input pedigree record integers ...\" to specify at least %s integers"
                                                         , E,N,Y,N}, /*ERF_NIN*/  
{" Expected \"input pedigree record integers ...\" to specify at least %s reals"
                                                         , E,N,Y,N}, /*ERF_NRL*/  
{" Incorrect number of traits selected for program \"%s\""      ,E,N,Y,N}, /*ERR_STN*/
{" Incorrect number of fixed effects selected for program \"%s\""
                                                                ,E,N,Y,N}, /*ERR_SFN*/
{" A trait number is > the number of real numbers in the pedigree file"
					                        ,E,N,N,N}, /*ERR_TRF*/
{" A fixed effect number is > the number of integers - 4 in the pedigree file"
                                                                ,E,N,N,N}, /*ERR_FFF*/
{" Starting values given for non-selected traits or non-selected fixed effects"
                                                                ,W,N,N,N}, /*ERR_SEL*/
{" Starting values must be given for the additive variances"    ,E,N,N,N}, /*ERR_VA0*/
{" Starting values must be given for the residual variances"    ,E,N,N,N}, /*ERR_VR0*/
{" Wrong number of starting values for additive variances"      ,E,N,N,N}, /*ERR_VSA*/
{" Wrong number of starting values for residual variances"      ,E,N,N,N}, /*ERR_VSR*/
{" Starting values for additive variances must be non-zero"     ,E,N,N,N}, /*ERR_VAN*/
{" Starting values for residual variances must be non-zero"     ,E,N,N,N}, /*ERR_VRN*/
{" Wrong no. of starting values for additive genetic covariances or correlations"
                                                                ,E,N,N,N}, /*ERR_CSA*/
{" Wrong no. of starting values for residual covariances or correlations" 
                                                                ,E,N,N,N}, /*ERR_CSR*/
{" Wrong no. of fit/no-fit indicators for additive genetic covariances or correlations" 
                                                                ,E,N,N,N}, /*ERR_CFA*/
{" Wrong no. of fit/no-fit indicators for residual covariances or correlations"
                                                                ,E,N,N,N}, /*ERR_CFR*/
{" Inconsistent independent groups in \"fit additive covariances/correlations\" statement"
					                        ,E,N,N,N}, /*ERR_CAF*/
{" Inconsistent independent groups in \"fit residual covariances/correlations\" statement"
					                        ,E,N,N,N}, /*ERR_CRF*/
{" Non-zero starting values for independent additive covariances ignored"
					                        ,W,N,N,N}, /*ERR_CAZ*/
{" Non-zero starting values for independent residual covariances ignored"
					                        ,W,N,N,N}, /*ERR_CRZ*/
{" Additive genetic covariance matrix is not positive definite" ,E,N,N,N}, /*ERR_PDA*/
{" Residual covariance matrix is not positive definite"         ,E,N,N,N}, /*ERR_PD1*/
{" \"compute eigenvalues\" overrides \"set eigenvalues\" statement"
   				 		                ,W,N,N,N}, /*ERR_ET1*/
{" Eigenvalues will be computed"                                ,W,N,N,N}, /*ERR_ET2*/
{" Command line specification of peeling sequence file \n"
 "       overrides \"input peeling sequence file\""             ,W,N,N,N}, /*ERR_PT1*/
{" Expected at least one \"simulate\" statement"                ,F,N,N,N}, /*ERD_SIM*/
{" Cannot specify simulation of both unnamed and named chromosomes"
                                                                ,F,N,N,N}, /*ERD_CHR*/
{" Marker map is missing %s"                                    ,E,N,Y,N}, /*ERD_NMM*/
{" No. of markers mapped %s is not the same as no. to be simulated"
                                                                ,E,N,Y,N}, /*ERD_NOM*/
{" Trait map is missing %s"                                     ,E,N,Y,N}, /*ERD_NTM*/
{" No. of traits mapped %s is not the same as no. to be simulated"
                                                                ,E,N,Y,N}, /*ERD_NOT*/
{" Marker no. in trait map %s > no. of markers to be simulated" ,E,N,Y,N}, /*ERD_MMX*/
{" Trait %s has more than 1 locus"                              ,E,N,Y,N}, /*ERD_LLL*/
{" Trait %s is specified as both mapped and unlinked"           ,E,N,Y,N}, /*ERD_LUL*/
{" Allele frequencies missing for %s:  defaults are .5 .5"      ,W,N,Y,N}, /*ERD_NML*/
{" Additive variance not given for trait %s:  default is 0.0"   ,W,N,Y,N}, /*ERD_TAV*/
{" Residual variance not given for trait %s:  default is 0.0"   ,W,N,Y,N}, /*ERD_TRV*/
{" Neither the additive nor residual variance is given for trait %s"
                                                                ,E,N,Y,N}, /*ERD_NAR*/
{" Sum of additive and residual variance for trait %s is 0.0"   ,E,N,Y,N}, /*ERD_SVR*/
{" Allele frequencies missing for trait %s:  defaults are .5 .5",W,N,Y,N}, /*ERD_NTL*/
{" Genotype means not given for trait %s:  defaults are 0. 0. 0." 
                                                                ,W,N,Y,N}, /*ERD_NGM*/
{" Marker to trait distance is > marker to marker distance for trait %s"
                                                                ,E,N,Y,N}, /*ERD_MTD*/
{" Congruential seed (1st seed) must be odd,\n"  
 " Tausworthe seed (2nd seed) must be non-zero"                 ,W,Y,N,N}, /*ERD_SEE*/
{" Overrides \"output trait seeds only\" statement"             ,W,Y,N,N}, /*ERD_OTS*/
{" Overrides \"output marker seeds only\" statement"            ,W,Y,N,N}, /*ERD_OMS*/
{" Default marker seeds are used"		                ,W,N,N,N}, /*ERD_SSD*/
{" Output pedigree file is not named"                           ,E,N,N,N}, /*ERD_OPF*/
{" No output seed file named.  \"output trait seeds only\" ignored" 
                                                                ,W,N,N,N}, /*ERD_OST*/
{" No output seed file named.  \"Output marker seeds only\" ignored"
                                                                ,W,N,N,N}, /*ERD_OSM*/
{" The number of fixed effect levels in the pedigree file \n"
 "     is > the maximum no. defined in limdefs.h"               ,F,N,N,N}, /*ERR_FFL*/
{" The number of starting values given for fixed effect levels\n"
 "     is not the same as the number of levels found in the pedigree file"
					                        ,W,N,N,N}, /*ERR_FFN*/
{" The largest component exceeds the maximum size permitted"
 " Rerun with the \"allow component size\" statement"	        ,F,N,N,N}, /*ERR_CHG*/
{" The number of eigenvalues in the \"set eigenvalues\" statement\n"
 "    is not the same as the number of observed individuals"    ,F,N,N,N}, /*ERR_EIP*/
{" Can't open eigenvalues file \"%s\""		 	        ,F,N,Y,N}, /*ERR_OEF*/
{" The number of eigenvalues in the eigenvalues file\n"
 "    is not the same as the number of observed individuals"    ,F,N,N,N}, /*ERR_EIE*/
{" The number of observed individuals exceeds the maximum number permitted\n"
 " Rerun with the \"allow observed individuals\" statement"     ,F,N,N,N}, /*ERR_OBS*/
{" Program quitting due to too many input errors"	        ,Q,N,N,N}, /*ERR_MAX*/
{" Program quitting due to fatal error"		                ,Q,N,N,N}, /*ERR_FAT*/
{" Program quitting due to errors in parameter statements"      ,Q,N,N,N}};/*ERR_QIT*/

#endif  /*ERRSPARS_H*/

