
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*       globs.h:  header file defining global variables               */
/*                 -  global in scope                                  */
/*                 -  included by main programs and function libraries */
/*                 -  contains modelling or simulation parameters      */
/*                    derived from the input statements                */ 
/*                 -  values are stored by Pars/ functions             */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef  GLOBS_H		
#define  GLOBS_H

/***  scalar variables, etc, derived from the parameter statements  ****/

GLOBAL int    PedSize;     

GLOBAL int    NInts,        NReals,       NFix,         NNInts;

GLOBAL int    NStdIn,       StdIn[5],     NStdOut,      StdOut[5];

GLOBAL int    DiscardReals;

GLOBAL int    NLoci,        NMarks,       NTraits,      NUnlnkd;

GLOBAL int    SexIn,        SexOp,        SexOut;

GLOBAL int    ObsIn,        ObsOp,        ObsOut;

GLOBAL int    AllVarOut,    ChronOut;

GLOBAL int    MarkSeedSet,  TraitSeedSet, MarkSeedsOut, TraitSeedsOut;

GLOBAL uint32 MarkSeedC,    MarkSeedT,    TraitSeedC,   TraitSeedT;

       enum  sEnums {NAM, PA, MA, SEX, OBS};

/*****     the following variables are used by Nghds/ functions     *****/

GLOBAL int    MaxNames;    		/*Largest name for individual   */

GLOBAL int    NColData;	        	/*(No. of integers) + 1 to      */
                                        /*selectively read from ped file*/

GLOBAL int    MaxNCmps;   		/*No. of components allocated	*/

GLOBAL int    SelTrait[FMAX_REAL];	/*Selection/nonselection of 	*/
					/*reals (traits) from ped file	*/

GLOBAL int    SelInt[FMAX_INT];   	/*Selection/nonselection of     */
					/*integers from ped file  	*/

 
/**** the following variables and tables are used by Quant functions ****/

GLOBAL int    FENo[FIXEFF];		/*For fixed effects selected,	*/
					/*fixed effect #		*/

GLOBAL int    TrNo[MAX_TRAIT];		/*For traits selected,		*/
					/*trait #			*/

GLOBAL int    FESex;                    /*Gender a fixed effect?        */

GLOBAL int    FfDex[MAX_TRAIT][FIXEFF];         /*Presence/absence of   */
                                        /*fixed effects j in trait i    */

GLOBAL int    IFix[FIXEFF]; 	    	/*Number of levels of fixed	*/
					/* effects 			*/

GLOBAL int    IOCor;			/*IOCor == TRUE:  output        */
                                        /*correlations                  */

GLOBAL int    ChkEigC;			/*flag for printing progress in	*/
					/*computing eigenvalues		*/
  

/*** the following variables are used by Quant/ and PolyEM functions ****/

GLOBAL double Mufti[MAX_FEP][FIXEFF][LEVELS+1];  /*Fixed effect params; */
                                        /*Up to MAX_FEP combinations    */

GLOBAL double SigAdd, SigErr;           /* variance parameters          */
                                        /* (univariate)                 */

GLOBAL double TauSq[MAX_TRAIT*(MAX_TRAIT+1)/2],    /* covariances	*/
              SigSq[MAX_TRAIT*(MAX_TRAIT+1)/2];


/*** the following structure, _filtab, is used by Stuff/ functions      */
/*   and by other MORGAN functions                                      */

#define PAR      "par"
#define PED      "ped"
#define EIGEN    "eig"
#define PEEL     "peel"
#define SEED     "seed"
#define OPED     "oped"
#define OSEED    "oseed"

struct _filtab  {
  char *type;			        /*file type (com. line keyword) */
  int command;			  	/*flag for named in command line*/
  int param;				/*flag for named in par statemnt*/
  int done;                             /*flag for file has been read   */
  int parstat;				/*flag for contains param stmnts*/
  char *pcods;				/*progs which use this file type*/
  char *fop;				/*fopen mode "r", "w" or "a"	*/
  char *fname;				/*file name			*/
  char *fdesc;				/*description of file		*/
                };
typedef struct _filtab * FILTAB_PTR;

#ifdef  DEFINE_DATA
  struct _filtab Filtab[] = {
  { PAR   , N, N, N, Y, "pubgmd", "r", NULL, "parameter file" },
  { PED   , N, N, N, Y, "pubgmd", "r", NULL, "input pedigree file" },
  { EIGEN , N, N, N, N, "bgm"   , "r", NULL, "eigenvalue file"},
  { PEEL  , N, N, N, N, "u"     , "r", NULL, "peeling file" },
  { SEED  , N, N, N, Y, "d"     , "r", NULL, "seed file" },
  { OPED  , N, N, N, N, "pubgmd", "w", NULL, "output pedigree file" },
  { OSEED , N, N, N, N, "d"     , "a", NULL, "output seed file" },
  { ""    , N, N, N, N, ""      , "",  NULL, "" }}; 
#else
  extern struct _filtab Filtab[];
#endif  /*DEFINE_DATA*/


/**** the following variables and tables are used by PolyEM programs ****/

                                        /*See limits.h for default	*/
					/*values                        */

GLOBAL int    ItMo;			/*Breeding-value iterations	*/
GLOBAL int    ItEM;			/*Number of EM iterations	*/
GLOBAL int    ItWt;			/*How often to write out estmts */
GLOBAL int    IPtEM;			/*iem =1 partitioned EM		*/
GLOBAL int    IEn;			/*ien =1 fit environmental model*/
GLOBAL double Diff;			/*Convergence criterion for	*/
					/*breeding values iteration	*/
GLOBAL int    ICh;			/*ich =1 -- call bigcheig	*/

GLOBAL int    ChkEig;			/*flag for printing eigenvals	*/
GLOBAL int    ChkGnv;			/*flag for printing G-iverse	*/
GLOBAL int    ChkGmt;			/*flag for printing G-matrix    */
GLOBAL int    ChkTrc;			/*flag for printing trace	*/

GLOBAL int    TauModel[MAX_TRAIT*(MAX_TRAIT+1)/2],      /* fit/no fit   */
              SigModel[MAX_TRAIT*(MAX_TRAIT+1)/2];      /* covariances  */
                                                        /* (multivar)   */

          
/******     the following structures are used by genedrop          ******/
                        
                        /* macros for recombMap->recombTyp below        */

#define FM 1			/* first marker (on chromosome)		*/
#define OM 2			/* other than first marker		*/
#define FT 3			/* trait is first locus on chromosome   */
#define IT 4			/* intermediate trait - has preceding	*/
				/* marker/trait locus & following marker*/
#define ET 5			/* trait with no following markers	*/
#define UT 6			/* unlinked trait			*/

			/*recombMap:  identifies neighbors & probs	*/
			/*            used in recombination algorithms	*/
struct recombMap {		
    int id;				/* markers & traits are in order*/
    int recombTyp;			/* of chrom, locus;  unlinked	*/
    int iA;				/* traits are last		*/
    int iB;
    double probA;
    double probB;
                 };
typedef struct recombMap RMAP;
typedef RMAP * RMAP_PTR;
GLOBAL RMAP_PTR PRecombMap NINIT;

			/*traitPars:  parameters used for computing	*/
   			/*            the additive and residual 	*/
    			/*            of the phenotype			*/
struct traitPars {
    int id;				/* ids are in recombMap order	*/
    double means[3];
    double addi_term_found;
    double addi_term;
    double resi_term;
                 };
typedef struct traitPars TPARS;
typedef TPARS * TPARS_PTR;
GLOBAL TPARS_PTR PTraitPars NINIT;

			/*traitOut:  traits in numerical order 		*/
struct traitOut {
    int id;
    int wid;
    int dec;
    int ixG;
    int ixPh;
                };
typedef struct traitOut TOUT;
typedef TOUT * TOUT_PTR;
GLOBAL TOUT_PTR PTraitOut NINIT;

			/*alleleFreq:  allele frequencies for each locus*/
struct alleleFreq{		
    int nAlleles;			/*no. of alleles at this locus	*/
    double * pCumFreq;			/*ptr to list of cumulative frqs*/
                 };		
typedef struct alleleFreq ALLELE;
typedef ALLELE * ALLELE_PTR;
GLOBAL ALLELE_PTR PAlleleFreq NINIT;

GLOBAL int WidAlle;                     /*no. characters for printing   */
                                        /*alleles                       */
GLOBAL double * peigen;

#endif  /*GLOBS_H*/

