
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*       parsepars.h:  header file for the Pars/ library               */
/*                     -  of file scope                                */ 
/*                     -  included by Pars/ functions                  */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef  PARSEPARS_H		
#define  PARSEPARS_H

#define MA1ST   1                       /*Mother comes 1st in record    */
#define PA1ST   2                       /*Father comes 1st in record    */

#define IEM     0			/*Type of EM:  EM=0, full EM	*/
					/*EM=1, partitioned EM		*/
#define IEN     0			/*IEN=1, fit an environmental	*/
					/*model as well			*/
#define ICH     0			/*Do not compute eigenvalues flg*/
#define IOC     0			/*Output correlations flag	*/

#define KEYWD 0                         /*Token is a keyword            */
#define PDAT 1                          /*Token is data                 */
#define MAXNKEY  6			/*The max # of keywords in a 	*/
					/*statement			*/
#define MAXNTOKEN 200			/*The max # of tokens in a 	*/
					/*statement (init. # only)	*/
#define NOMTOKLEN  8			/*Nominal # of characters per	*/
					/*token in an input statement	*/

#define COM_LIN_FORM \
 "<prognam> <parfile> [ped <pedfile>] [eigen <eigenfile>] [peel <peelfile>]"

#define SIGSQ(r, c)  SigSq[(r) * (r - 1) /2 + c - 1]
#define TAUSQ(r, c)  TauSq[(r) * (r - 1) /2 + c - 1]
#define SIG_MODL(r, c)  SigModel[(r) * (r - 1) /2 + c - 1]
#define TAU_MODL(r, c)  TauModel[(r) * (r - 1) /2 + c - 1]

static FILTAB_PTR  pfloop, pft;
static FILE * pfile;

/*  line_in[]   - the next line of input data				*/
/*  token[][]   - the tokens found in the input statemnt		*/
/*  token[]     - a list of pointers to the start of each token		*/
/*  tok_typ[]   - corresponding token type 				*/
/*                (alphabetic, numeric or string)			*/
/*  aggre_key[] - the aggregate key					*/ 
/*		  (concatenated first 4 characters of each keyword)	*/
/*  key_ix      - the statement type, as determined by the aggregate key*/

static char     line_in[MAXLINLEN+1];
static char  ** token;
static int    * tok_typ;
static char     aggre_key[MAXNKEY*4+1];
static int      key_ix;


/*  Limits for each program 						*/

static struct _prog  {
  char *pnam;				/*program name			*/
  char pcod; 				/*one char program code		*/
  int  min_trait; 			/*minimum no. of traits		*/
  int  max_trait; 			/*maximum no. of traits		*/
  int  max_fef;				/*maximum no. of fixed effects	*/
  int  max_chrom;                       /*maximum no. of chromosomes    */
  int  max_mark;			/*maximum no. of markers	*/
  int  max_allele;			/*maximum no. of alleles	*/
  int  max_ped; 			/*maximum pedigree size		*/
  int  max_obs;				/*maximum no. observed		*/
  int  max_cmp;				/*maximum component size	*/
               } *pmx; 

static struct _prog prog[] = {
 {"univar"  , 'u',  1,         1,   FIXEFF,       0,         0,          0, 
                                          MAX_U_PED, MAX_U_OBS,          0 },
 {"unibig"  , 'g',  1,         1,   FIXEFF,       0,         0,          0,
                                            MAX_PED,   MAX_OBS,  MAX_CPSIZ },
 {"bivar"   , 'b',  2,         2,   FIXEFF,       0,         0,          0,
                                            MAX_PED,   MAX_OBS,  MAX_CPSIZ },
 {"multivar", 'm',  1, MAX_TRAIT,   FIXEFF,       0,         0,          0,
                                            MAX_PED,   MAX_OBS,  MAX_CPSIZ },
 {"genedrop", 'd',  0, MAX_SIM_TR,       0,MAX_CHROM, MAX_MARK,  MAX_ALLEL,
                                            MAX_PED,         0,          0 },
 {"pedcheck", 'p',  0,         0,        0,       0,         0,          0,
                                            MAX_PED,         0,          0 },
 {""        ,'\0',  0,         0,        0,       0,         0,          0,
                                                  0,         0,          0 } };

/*  function prototypes for functions which correspond to the various	*/
/*  input statement types						*/

static int
    not_impl      (void),
    allow_cmpsiz  (void),
    allow_maxnam  (void),
    allow_obsiz	  (void),
    allow_pedsiz  (void),
    assign_gend   (void),
    assum_all_obs (void),
    check_eigen	  (void),
    check_eig_cmp (void),
    check_ginv	  (void),
    check_gmat    (void),
    check_trace	  (void), 
    comp_eigen    (void),
    fit_addi	  (void),
    fit_no_gen    (void),
    fit_resi      (void),
    ignor_gend    (void),
    in_eig_fil	  (void),
    in_gend_abs   (void),
    in_gend_pres  (void),
    in_ord_fath   (void),
    in_ord_moth   (void),
    in_obs_abs    (void),
    in_obs_pres   (void),
    in_ped_fil	  (void),
    in_ped_int    (void),
    in_ped_int_rl (void),
    in_pedsiz     (void),
    in_peel_fil	  (void),
    in_seed_fil   (void),
    label_mark    (void),
    label_trait   (void),
    lim_breed     (void),
    lim_em_iter   (void),
    map_cmarkd    (void),
    map_cmarkp    (void),
    map_cmarkr    (void),
    map_ctraitd   (void),
    map_ctraitr   (void),
    map_markd     (void),
    map_markp     (void),
    map_markr     (void),
    map_traitd    (void),
    map_traitr    (void),
    out_all       (void),
    out_chron     (void),
    out_corr	  (void),
    out_covar     (void),
    out_mark_seeds(void),
    out_ped_fil   (void),
    out_ord_fath  (void),
    out_ord_moth  (void),
    out_seed_fil  (void),
    out_space     (void),
    out_trait_seeds(void),
    sel_trait     (void),
    sel_tr_eff    (void),
    set_addi_var  (void),
    set_br_conv   (void),
    set_cmark_frq (void),
    set_eigen     (void),
    set_geno_mean (void),
    set_mark_frq  (void),
    set_mark_seeds(void),
    set_resi_var  (void),
    set_trait_frq (void),
    set_trait_seeds(void),
    sim_cmark     (void),
    sim_cmark_trait(void),
    sim_mark      (void),
    sim_mark_trait(void),
    sim_unlinked  (void),
    start_add_cor (void),
    start_add_cov (void),
    start_add_var (void),
    start_effect  (void),
    start_mean    (void),
    start_res_cov (void),
    start_res_cor (void),
    start_res_var (void),
    titl_top      (void),
    use_part_em   (void),
    use_full_em   (void);
 
/*  prototype for input statement 					*/
/*  proto.key[]:  string containing the first four letters of each 	*/
/*                keyword in the statement.				*/
/*  proto.par:    a code for each data value or list of values in the	*/
/*                statement.  						*/
/*	          "i" - an integer value, "k" - a list of integer values*/
/*                "r" - a real value,     "x" - a list of real values 	*/
/*  proto.func:   the name of the function which acts upon the data 	*/
/*                from the statement   			                */
/*  proto.count:  the number of times this statement type occurs in the */
/*                input file						*/


static struct _proto {				
       char * key;
       char * par;  
       char * pcods;
       int (* func) (void);
       int    count;
               } *ppp;

static struct _proto  proto[] = {
{"allo""comp""size"                 , "i"     , "ugbm   " , allow_cmpsiz  , 0},
{"allo""maxi""name"		    , "i"     , "ugbm   " , allow_maxnam  , 0},
{"allo""obse""indi"	            , "i"     , "ugbm   " , allow_obsiz   , 0},
{"allo""pedi""size"		    , "i"     , "ugbmdp " , allow_pedsiz  , 0},
{"assi""gend"                       , ""      , "ugbmdp " , assign_gend   , 0},
{"assu""all" "obse"                 , ""      , "    dp " , assum_all_obs , 0},
{"chec""eige"			    , ""      , "ugbm   " , check_eigen   , 0},
{"chec""eige""comp"		    , ""      , "ugbm   " , check_eig_cmp , 0},
{"chec""ginv"			    , ""      , " gbm   " , check_ginv    , 0},
{"chec""gmat"			    , ""      , "u      " , check_gmat    , 0},
{"chec""trac"			    , ""      , "ugbm   " , check_trace   , 0},
{"comp""eige"			    , ""      , "ubgm   " , comp_eigen    , 0}, 
{"fit" "addi""corr"                 , "k"     , "   m   " , fit_addi      , 0},
{"fit" "addi""cova"                 , "k"     , "   m   " , fit_addi      , 0},
{"fit" "envi""mode"		    , ""      , "   m   " , fit_no_gen    , 0},
{"fit" "resi""corr"		    , "k"     , "   m   " , fit_resi      , 0},
{"fit" "resi""cova"		    , "k"     , "   m   " , fit_resi      , 0},
{"igno""gend"                       , ""      , "ugbmdp " , ignor_gend    , 0},
{"inpu""eige""file"		    , "s"     , " gbm   " , in_eig_fil    , 0},
{"inpu""pedi""file"		    , "s"     , "ugbmdpa" , in_ped_fil    , 0},
{"inpu""pedi""reco""fath""moth"     , ""      , "ugbmdp " , in_ord_fath   , 0},
{"inpu""pedi""reco""gend""abse"     , ""      , "ugbmdp " , in_gend_abs   , 0},
{"inpu""pedi""reco""gend""pres"     , ""      , "ugbmdp " , in_gend_pres  , 0},
{"inpu""pedi""reco""inte"           , "i"     , "    dpa" , in_ped_int    , 0},
{"inpu""pedi""reco""inte""real"     , "ii"    , "ugbmdpa" , in_ped_int_rl , 0},
{"inpu""pedi""reco""moth""fath"	    , ""      , "ugbmdp " , in_ord_moth   , 0},
{"inpu""pedi""reco""obse""abse"     , ""      , "ugbmdp " , in_obs_abs    , 0},
{"inpu""pedi""reco""obse""pres"     , ""      , "ugbmdp " , in_obs_pres   , 0},
{"inpu""pedi""size"                 , "i"     , "ugbmdpa" , in_pedsiz     , 0},
{"inpu""peel""sequ""file"           , "s"     , "u      " , in_peel_fil   , 0},
{"inpu""seed""file"                 , "s"     , "    d  " , in_seed_fil   , 0},
{"labe""chrom""mark"                , "iis"   , "    d  " , not_impl      , 0},
{"labe""mark"                       , "is"    , "    d  " , not_impl      , 0},
{"labe""trai"                       , "is"    , "ugbmd  " , not_impl      , 0},
{"limi""bree""iter"		    , "i"     , "ugbm   " , lim_breed     , 0},
{"limi""em"  "iter"		    , "i"     , "ugbm   " , lim_em_iter   , 0},
{"map" "chro""mark""dist"           , "ix"    , "    d  " , map_cmarkd    , 0}, 
{"map" "chro""mark""posi"           , "ix"    , "    d  " , map_cmarkp    , 0},
{"map" "chro""mark""reco""frac"     , "ix"    , "    d  " , map_cmarkr    , 0},
{"map" "chro""trai""mark""dist"     , "ikkx"  , "    d  " , map_ctraitd   , 0},
{"map" "chro""trai""mark""reco""frac","ikkx"  , "    d  " , map_ctraitr   , 0},
{"map" "mark""dist"                 , "x"     , "    d  " , map_markd     , 0},
{"map" "mark""posi"                 , "x"     , "    d  " , map_markp     , 0},
{"map" "mark""reco""frac"           , "x"     , "    d  " , map_markr     , 0},
{"map" "trai""mark""dist"           , "kkx"   , "    d  " , map_traitd    , 0},
{"map" "trai""mark""reco""frac"     , "kkx"   , "    d  " , map_traitr    , 0},
{"outp""mark""seed""only"           , ""      , "    d  " , out_mark_seeds, 0},
{"outp""pedi""chro"                 , ""      , "    dp " , out_chron     , 0},
{"outp""pedi""file"		    , "s"     , "ugbmdp " , out_ped_fil   , 0},
{"outp""pedi""reco""fath""moth"     , ""      , "    dp " , out_ord_fath  , 0},
{"outp""pedi""reco""moth""fath"     , ""      , "    dp " , out_ord_moth  , 0},
{"outp""pedi""vari""all"            , ""      , "    d  " , out_all       , 0},
{"outp""seed""file"		    , "s"     , "    d  " , out_seed_fil  , 0},
{"outp""spac""em"  "iter"           , "i"     , "ugbm   " , out_space     , 0},
{"outp""stat""corr"                 , ""      , "   m   " , out_corr      , 0},
{"outp""stat""cova"                 , ""      , "  bm   " , out_covar     , 0},
{"outp""trai""seed""only"           , ""      , "    d  " , out_trait_seeds,0},
{"sele""trai"                       , "i"     , "ugbm   " , sel_trait     , 0},
{"sele""trai""effe"		    , "ik"    , "ugbm   " , sel_tr_eff    , 0},
{"set" "addi""corr"                 , ""      , "    d  " , not_impl      , 0},
{"set" "addi""cova"                 , ""      , "    d  " , not_impl      , 0},
{"set" "bree""conv"		    , "r"     , "ugbm   " , set_br_conv   , 0},
{"set" "chro""mark""freq"           , "ikx"   , "    d  " , set_cmark_frq , 0},
{"set" "eige"                       , "x"     , " gbm   " , set_eigen     , 0},
{"set" "mark""freq"                 , "kx"    , "    d  " , set_mark_frq  , 0},
{"set" "mark""seed"                 , "hh"    , "    d  " , set_mark_seeds, 0},
{"set" "peel""seq"		    , "x"     , ""        , not_impl      , 0},
{"set" "resi""corr"                 , ""      , "    d  " , not_impl      , 0},
{"set" "resi""cova"                 , ""      , "    d  " , not_impl      , 0},
{"set" "trai""addi""vari"           , "kr"    , "    d  " , set_addi_var  , 0},
{"set" "trai""freq"                 , "kx"    , "    d  " , set_trait_frq , 0},
{"set" "trai""geno""mean"           , "kx"    , "    d  " , set_geno_mean , 0},
{"set" "trai""resi""vari"           , "kr"    , "    d  " , set_resi_var  , 0},
{"set" "trai""seed"                 , "hh"    , "    d  " , set_trait_seeds,0},
{"simu""chro""mark"                 , "ii"    , "    d  " , sim_cmark     , 0},
{"simu""chro""mark""trai"           , "iii"   , "    d  " , sim_cmark_trait,0},
{"simu""mark"                       , "i"     , "    d  " , sim_mark      , 0},
{"simu""mark""trai"                 , "ii"    , "    d  " , sim_mark_trait, 0},
{"simu""unli""trai"                 , "k"     , "    d  " , sim_unlinked  , 0},
{"star""addi""corr"                 , "x"     , "  bm   " , start_add_cor , 0},
{"star""addi""cova"		    , "x"     , "  bm   " , start_add_cov , 0},
{"star""addi""vari"                 , "x"     , "ugbm   " , start_add_var , 0},
{"star""resi""corr"		    , "x"     , "  bm   " , start_res_cor , 0},
{"star""resi""cova"                 , "x"     , "  bm   " , start_res_cov , 0},
{"star""resi""vari"		    , "x"     , "ugbm   " , start_res_var , 0},
{"star""trai""effe"		    , "iix"   , "ugbm   " , start_effect  , 0},
{"star""trai""mean"                 , "ir"    , "ugbm   " , start_mean    , 0},
{"titl"                             , "s"     , "ugbmdp " , not_impl      , 0},
{"use" "part" "em" 		    , ""      , "   m   " , use_part_em   , 0},
{"use" "full" "em"	            , ""      , "   m   " , use_full_em   , 0},
{""			            , ""      , ""        , not_impl      ,0}};


/*  descriptions of input statements, SAME ORDER and SAME NUMBER	*/	
/*  as the statement prototypes above					*/

static char * input_statement[] = {
	"allow component size  I"			,
	"allow maximum name  I"				,
	"allow observed individuals  I"			,
        "allow pedigree size  I"			,
        "assign gender"                                 ,
        "assume all observed"                           , 
        "check eigenvalues"				,
        "check eigenvalue computation"			,
        "check ginverse"				,
        "check gmatrix"					,
        "check trace"					,
        "compute eigenvalues"				,	
	"fit additive correlations  K12 ..."		,
	"fit additive covariances  K12 ..."		,
	"fit environmental model"		        ,
	"fit residual correlations  K12 ..."		,
	"fit residual covariances  K12 ..."		, 
        "ignore gender"                                 ,
        "input eigenvalue file S"			,
        "input pedigree file  S"			,
        "input pedigree record father mother"	        ,	
        "input pedigree record gender absent"           ,
        "input pedigree record gender present"          ,
        "input pedigree record integers  I"             ,
	"input pedigree record integers  I  reals  J"	,
        "input pedigree record mother father"		,
        "input pedigree record observed absent"         ,
        "input pedigree record observed present"        ,
	"input pedigree size  I"			,
	"input peeling sequence file  S"		,
        "input seed file  S"                            ,
        "label chromosome  I  marker  J  S"             ,
        "label marker  J  S"                            , 
        "label trait  I  S"                             ,
        "limit breeding iterations  I"			,
	"limit EM iterations  I"			,
        "map chromosome  I  marker distances  X1 ..."   ,
        "map chromosome  I  marker positions  X1 ..."   ,
        "map chromosome  I  marker "
                               "recombination fractions  X1 ..." ,
        "map chromosome  I  traits  J1 ...  markers  K1 ...  "
                                             "distances  X1 ..." ,
        "map chromosome  I  traits  J1 ...  markers  K1 ...  " 
                               "recombination fractions  X1 ..." ,
        "map marker distances  X1 ..."                  ,
        "map marker positions  X1 ..."                  ,
        "map marker recombination fractions  X1 ..."    ,
        "map traits  J1 ...  markers  K1 ...  distances  X1 ..." ,
        "map traits  J1 ...  markers  K1 ...  "
                               "recombination fractions  X1 ..." ,
        "output marker seeds only"                      ,
        "output pedigree chronological"                 ,
        "output pedigree file  S"                       ,
        "output pedigree record order father mother"    ,
        "output pedigree record order mother father"    ,
        "output pedigree variables all"                 ,
        "output seed file  S"				,
	"output spacing  I  EM iterations"		,
	"output statistics correlations"		,
	"output statistics covariances"			,
        "output trait seeds only"                       ,
	"select trait  I"				,
	"select trait  I  effects  J1 ..."		,
        "set additive correlations  X12 ..."		,
        "set additive covariances  X12 ..."             ,
        "set breeding convergence  X"			,
        "set chromosome  I  markers  K1 ...  frequencies  X1 ... " ,
	"set eigenvalues  X1 ..."			,
        "set markers  K1 ...  frequencies  X1 ... "     ,
        "set marker seeds  H1  H2"                      ,
        "set peeling sequence  X1 ..."                  ,
        "set residual correlations  X12 ..."            ,
        "set residual covariances  X12 ..."		,
        "set traits  J1 ...  additive variances  X1 ..." ,
        "set traits  J1 ...  frequencies  X1 ..."       ,
        "set traits  J1 ...  genotype means  X1 ..."    ,
        "set traits  J1 ... residual variances  X1 ..." ,
        "set trait seeds  H1  H2"                       ,
        "simulate chromosome  I  markers  J"            ,
        "simulate chromosome  I  markers  J  traits  K" ,
        "simulate markers  J"                           ,
        "simulate markers  J  traits  K"                ,
        "simulate unlinked traits  K1 ..."              ,
	"start additive correlations  X12 ..."		,
	"start additive covariances  X12 ..."		,
	"start additive variances  X1 ..."		,
	"start residual correlations  X12 ..."		,
	"start residual covariances  X12 ..."		,
	"start residual variances  X1 ..."		,
	"start trait  I  effect  J  X1 ..."		,
	"start trait  I  mean  X"			,
        "title  S"                                      ,
	"use partitioned EM"				,
	"use full EM"                                   ,
	""                                               } ;  

/*  these words may begin a new input statement, thus delimiting the	*/	
/*  previous statement.							*/

static char * verb[] = {
             "allo",
             "assi",
             "assu",
             "chec",
             "comp",
	     "fit",
             "igno",
	     "inpu",
             "labe",
	     "limi",
             "map",
	     "outp",
	     "sele",
	     "set",
             "simu",
	     "star",
             "titl",
	     "use",
             "****",
	     ""     };

 
/***    limits overriding variables                                   ***/
 
static int    allw_pedsiz,  allw_cmpsiz,   allw_obsiz,   allw_maxnam;
 
/***    record variables                                              ***/

static int parentOrdIn, parentOrdOut;

/***    the following macro constants are used by proc_pars() and     ***/
/***    proc_drop()                                                   ***/

#define NC 0        /* no chromosome     */
#define C  1        /* chromosome        */

#define NO_CH_NO -1 /* chrom not needed  */

#define NT 0        /* no traits         */
#define T  1        /* traits            */

#define R  0        /* recomb fraction   */
#define D  1        /* distance          */
#define P  2        /* position          */

#define MD  0       /* marker data       */
#define TD  1       /* trait data        */

#define AV  0	    /* additive variance */
#define RV  1       /* residual variance */

#define NIL -1      /* ignore            */


/***    the following data structures are used by proc_pars() and     ***/
/***    proc_drop() for genedrop simulation specifications            ***/

struct unlnkd_node {                    /*Unlnkd_node:  unlinked traits */
    int id;
    struct unlnkd_node * p_next;
                   };
typedef struct unlnkd_node UNLNKD;
typedef UNLNKD * UNLNKD_PTR;
static UNLNKD_PTR first_unlnkd_node = NULL;


struct sim_node {       		/*Sim_node:  no. of markers and */
    int chrom_no;                       /*no. of linked traits simulated*/
    int no_mark;                        /*on this chromosome            */
    int no_traits;
    struct sim_node * p_next;
                 };
typedef struct sim_node SIM;
typedef SIM * SIM_PTR;
static SIM_PTR first_sim_node = NULL;


struct map_node {			/*Map_node:  no. of markers or 	*/
    int chrom_no;			/*traits on this chromosome and */
    int no_loci;			/*pointer to list of distances  */
    struct dist_list * p_dist;    	/*for these loci                */
    struct map_node * p_next;         
                 };
typedef struct map_node MAP;
typedef MAP * MAP_PTR;
static MAP_PTR first_mark_map_node = NULL;
static MAP_PTR first_trait_map_node = NULL;

struct dist_list {			/*Dist_node:  distance of marker*/
    int id;                             /*or trait from preceding marker*/
    int mark_id;                        /*on this chromosome            */
    double dist;     
    double prob;
                    };
typedef struct dist_list DIST;
typedef DIST * DIST_PTR;


struct locus_node {                     /*Locus_node:  the number of    */
    int chrom_no;			/*alleles for this loci	and a   */
    int id;				/*pointer to the allele 	*/
    int no_alleles;			/*frequencies			*/
    struct freq_node * p_freq_node;
    struct locus_node * p_next;
                    }; 
typedef struct locus_node LOCUS;
typedef LOCUS * LOCUS_PTR;
static LOCUS_PTR first_mark_locus_node = NULL;
static LOCUS_PTR first_trait_locus_node = NULL;

struct freq_node {
    int no_loci;
    double * p_freq;
                 };
typedef struct freq_node FREQ;
typedef FREQ * FREQ_PTR;


struct means_node {			/*Means_node: the genotype      */
    int id;				/*means (3) for this trait      */
    double mean[3];
    struct means_node * p_next;
                   };
typedef struct means_node MEANS;
typedef MEANS * MEANS_PTR;
static MEANS_PTR first_means_node = NULL;


struct vari_node {			/*Vari_node:  additive or       */
    int id;		        	/*residual variance for the  	*/
    double vari;			/*trait                         */
    struct vari_node * p_next;
                  };
typedef struct vari_node VARI;
typedef VARI * VARI_PTR;
static VARI_PTR first_addi_node = NULL;
static VARI_PTR first_resi_node = NULL;


/***  the following variables and tables, used by proc_pars(),        ***/
/***  proc_poly() and check_pars(), contain PolyEM modelling          ***/
/***  specifications                                                  ***/

static int    add_cor_flag,      	/*Starting values are correla-	*/
              res_cor_flag;             /*tions, not covariances	*/

static int    naddvar, naddcov,         /*No. starting values input	*/
              nresvar, nrescov;         /*for variances and covariances	*/

static int    nfitadd, nfitres; 	/*No. fit flags for covariances	*/

static int    seltrff[FMAX_REAL][FMAX_FEF];    /*Selection/nonselection */   
                                        /*of traits & fixed effects.    */
                                        /*Same as ffdex[][], but not    */
                                        /*condensed for traits selected */
                                   
static int    nstartff[FMAX_REAL][FMAX_FEF];   /*No. starting values for*/
                                        /*levels for the fixed effect   */
                                        /*(including mean) for the trait*/

static double startff[FMAX_REAL][FMAX_FEF][LEVELS];    /*Starting values*/
                                        /*for fixed effect levels/means */ 

static int    selff[FMAX_FEF];	      	/*Selection/nonselection of	*/
					/*fixed effects (incl mean)     */

static int    eigen_set, eigen_compute; /*Eigenvalue options		*/

static int    n_eigen;                  /*No. eigenvalues given         */


#endif  /*PARSEPARS_H*/

 
