 
#include "parsepoly.h"

/*
 * proc_poly() has as its input the results of parsing the parameter
 * statements by proc_pars() for the polyEM programs.  It checks this
 * data for consistency and completeness and, if there are no errors,
 * builds data structures used by univar, unibig, bivar or multivar.
 *							  MAJ Jul 1999 
 */


int proc_poly(void)
{
                                        /* checks:                      */
    if (!test_select())		        /* traits and fixed effects     */
        if (proc_err())
            return FALSE;
    if (!test_startff())		/* and starting values for fef's*/
        if (proc_err())
            return FALSE;
    if (!test_start_var())		/* and starting values for 	*/
       if (proc_err())		        /* variances and covariances	*/
            return FALSE;
    if (!test_eigen())		        /* and eigenvalue specifications*/
        if (proc_err())
            return FALSE;
    if (!test_peeling())		/* and peeling sequence specs   */
        if (proc_err())
            return FALSE;

    return TRUE;
}

/* ----- test fncs:  test results of processing input statements ------ */

/* -------------------- test functions for polyem --------------------- */


int test_select(void)			/* checks traits and fixed 	*/
{					/* effects selected, and stores */
    int i, j, m, n;			/* this info in arrays		*/

    for (i = 0, NTraits = 0; i < FMAX_REAL; ++i)
	if (SelTrait[i])
	{
	    if (NTraits < pmx->max_trait)
		TrNo[NTraits] = i + 1;  /* TrNo[]:  trait #		*/
	    ++NTraits;
	    seltrff[i][0] = TRUE;	/* (mean)		 	*/
	    selff[0] = TRUE;
	    for (j = 1; j < FMAX_FEF; ++j)
		if (seltrff[i][j])
		{
		    SelInt[j + 2] = TRUE;
		    selff[j] = TRUE;	/* selff[]:  fixed effects to be*/
		}			/* modelled		        */
	}
    SelInt[0] = 1;			/* SelInt[]:  data[][]          */
                                        /* subscripts                   */

    if (parentOrdIn == UNDEF || parentOrdIn == PA1ST)
    {
	SelInt[1] = 2;			/* (father)			*/
	SelInt[2] = 3;			/* (mother)			*/
    } else
    {
	SelInt[1] = 3;			/* (mother)			*/
	SelInt[2] = 2;			/* (father)			*/
    }
    SelInt[3] = 4;			/* (sex)			*/

    for (j = 4, n = 4; j < FMAX_INT; ++j)
	if (SelInt[j])
	{
	    ++n;
	    SelInt[j] = n;		/* (fixed effects)		*/
	}
    NColData = n + 1;

    if (NTraits < pmx->min_trait || NTraits > pmx->max_trait)
    {
	err_typ = ERR_STN;
        err_txt = pmx->pnam;
    }
    if (!err_typ)
    {
	for (j = 1, NFix = 0; j < FMAX_FEF; ++j)
	    if (selff[j])
	    {
		++NFix;
		if (NFix <= FIXEFF)
		    FENo[NFix] = j;	/* FENo[]:  fixed effect #	*/
	    }
	if (NFix + 1 > pmx->max_fef)
        {
	    err_typ = ERR_SFN;
            err_txt = pmx->pnam;
        }
    }
    if (!err_typ)
    {
	for (i = 0, m = 0; i < FMAX_REAL; ++i)
	    if (SelTrait[i])
	    {
		for (j = 0, n = 0; j < FMAX_FEF; ++j)
		    if (selff[j])
		    {
			if (m <pmx->max_trait && n < FIXEFF)
			    FfDex[m][n] = seltrff[i][j];
			++n;		/* FfDex[][]: fixed effects	*/
		    }			/* selected for traits	        */
		++m;
	    }
    }
    if (!err_typ)
	for (i = 0; i < FMAX_REAL; ++i)
	    if (SelTrait[i])
		if (i > NReals - 1)
		    err_typ = ERR_TRF;

    if (!err_typ)
	for (j = 4; j < FMAX_INT; ++j)
	    if (SelInt[j])
		if (j > NInts - 1)
		    err_typ = ERR_FFF;
    if (!err_typ)
        if (selff[1])			/* checks for gender a fixed	*/
            FESex = TRUE;	        /* effect                       */
        else
            FESex = FALSE;

    if (!err_typ)
	return TRUE;
    else
	return FALSE;
}

int test_startff(void)			/* checks for starting values 	*/
{					/* for fixed effects, and stores*/
    int i, j, k, n, m;			/* this info in Mufti[][][]	*/

    for (i = 0, m = 0; i < FMAX_REAL; ++i)
    {
	for (j = 0, n = 0; j < FMAX_FEF; ++j)
	{
	    if (nstartff[i][j])
	    {
		if (!seltrff[i][j])
		    err_typ = ERR_SEL;
		else
		    for (k = 0; k < LEVELS; ++k)
			Mufti[m][n][k + 1] = startff[i][j][k];
	    }
	    if (selff[j])
		++n;
	}
	if (SelTrait[i])
	    ++m;
    }
    if (!err_typ)
	return TRUE;
    else
	return FALSE;
}

int test_start_var(void)		/* checks variances 		*/
{
    int i, j, row, col, ngroups, group[MAX_TRAIT + 1];
    double x[MAX_TRAIT * (MAX_TRAIT + 1) / 2];

					/* variance checks:		*/
    if (naddvar == 0)			/* checks for starting values	*/
        err_typ = ERR_VA0;
    if (!err_typ)
        if (nresvar == 0)
            err_typ = ERR_VR0;

    if (!err_typ)			/* checks # of starting values	*/
        if (naddvar != NTraits)
            err_typ = ERR_VSA;
    if (!err_typ)
        if (nresvar != NTraits)
	    err_typ = ERR_VSR;

    if (!err_typ)			/* checks for non-zero starting	*/
        for (i = 1; i <= NTraits; ++i)			     /* values	*/
            if (SIGSQ(i, i) == 0.0)
                err_typ = ERR_VAN;
    if (!err_typ)
        for (i = 1; i <= NTraits; ++i)
            if (TAUSQ(i, i) == 0.0)
                err_typ = ERR_VRN;

                                        /* covariance checks:		*/
    if (!err_typ)			/* checks # starting values	*/
        if (naddcov > 0 && naddcov != NTraits * (NTraits - 1) / 2)
	    err_typ = ERR_CSA;
    if (!err_typ)
        if (nrescov > 0 && nrescov != NTraits * (NTraits - 1) / 2)
	    err_typ = ERR_CSR;

    if (!err_typ)			/*converts correlations to 	*/
        if (add_cor_flag)		/*covariances			*/
        {			
            for (row = 2; row <= NTraits; ++row)
                for (col = 1; col < row;  ++col)
		    if (SIGSQ(row, row) >= 0.0 && SIGSQ(col, col) >= 0.0)
			SIGSQ(row, col) *= sqrt(SIGSQ(row, row))
                                           * sqrt(SIGSQ(col, col));
		    else 
			SIGSQ(row, col) = 0.0;
	}
    if (!err_typ)	
        if (res_cor_flag)
        {			
            for (row = 2; row <= NTraits; ++row)
                for (col = 1; col < row;  ++col)
		    if (TAUSQ(row, row) >= 0.0 && TAUSQ(col, col) >= 0.0)
			TAUSQ(row, col) *= sqrt(TAUSQ(row, row)) 
                                           * sqrt(TAUSQ(col, col));
		    else 
			TAUSQ(row, col) = 0.0;
	}

					/* fit flags:			*/
    if (!err_typ)			/* checks # flags		*/
        if (nfitadd > 0 && nfitadd != NTraits * (NTraits - 1) / 2)
            err_typ = ERR_CFA;
    if (!err_typ)
        if (nfitres > 0 && nfitres != NTraits * (NTraits - 1) / 2)
            err_typ = ERR_CFR;		

    ngroups = 0;
    if (!err_typ)			/* checks independent groups for*/
	for (i = 1; i <= NTraits; ++i)	/* consistency			*/
        {
            group[i] = 0;
	    for (j = 1; j < i; ++j)
		if (SIG_MODL(i, j) != 0)
		    group[i] = group[j];
	    if (group[i] != 0)
            {
                for (j = 1; j < i; ++j)
		    if (group[i] == group[j])
                    {
			if (SIG_MODL(i, j) != 1)
                            err_typ = ERR_CAF;
                    }
                    else
                    {
			if (SIG_MODL(i, j) != 0)
                            err_typ = ERR_CAF;
                    }
	    }
	    else
            {
                ngroups = ngroups + 1;
		group[i] = ngroups;
 	    }	
	}
    if (!err_typ)	
	for (i = 1; i <= NTraits; ++i)
        {
            group[i] = 0;
	    for (j = 1; j < i; ++j)
		if (TAU_MODL(i, j) != 0)
		    group[i] = group[j];
	    if (group[i] != 0)
            {
                for (j = 1; j < i; ++j)
		    if (group[i] == group[j])
                    {
			if (TAU_MODL(i, j) != 1)
                            err_typ = ERR_CRF;
                    }
                    else
                    {
			if (TAU_MODL(i, j) != 0)
                            err_typ = ERR_CRF;
                    }
	    }
	    else
            {
                ngroups = ngroups + 1;
		group[i] = ngroups;
 	    }	
	}

    if (!err_typ)			/* checks for starting values 	*/
	for (i = 1; i <= NTraits; ++i)	/* of 0. for independent covars	*/
        {
	    for (j = 1; j < i; ++j)
		if (SIG_MODL(i, j) == 0 && SIGSQ(i, j) != 0.0)
                    err_typ = ERR_CAZ;
	}
    if (!err_typ)			
	for (i = 1; i <= NTraits; ++i)
        {
	    for (j = 1; j < i; ++j)
		if (TAU_MODL(i, j) == 0 && TAUSQ(i, j) != 0.0)
                    err_typ = ERR_CRZ;
	}	
    if (!err_typ)			/* checks for positive definite */
                                        /*matrices                      */
	if (pos_def_test(SigSq, x, NTraits))	
  	    err_typ = ERR_PDA; 
    if (!err_typ)
	if (pos_def_test(TauSq, x, NTraits))
            err_typ = ERR_PDR;

    if (pmx->max_trait == 1)		/* for univar & unibig, moves	*/
    {				        /* covar's to SigAdd & SigErr */
        SigAdd = SigSq[0];		
        SigErr = TauSq[0];
    }
 
    if (!err_typ)
	return TRUE;
    else
	return FALSE;
}

#define BB(b,i,j) ((b)[i*(i+1)/2+j])

/* code taken from chfact in multivar	*/

int pos_def_test(double *a, double *b, int n)
{					
    int i, j, k, kk;
    double zz, piv;

    kk = n * (n + 1) / 2;
    for (i = n - 1; i >= 0; i--)
    {
	zz = a[--kk];
	for (k = i + 1; k < n; k++)
	    zz -= BB(b, k, i) * BB(b, k, i);
	if (zz <= 0.0)
	    return TRUE;
	b[kk] = piv = sqrt(zz);
	for (j = i - 1; j >= 0; j--)
	{
	    zz = a[--kk];
	    for (k = i + 1; k < n; k++)
		zz -= BB(b, k, i) * BB(b, k, j);
	    b[kk] = zz / piv;
	}
    }
    return FALSE;
}

int test_eigen(void)			
{				
		
    if (eigen_set && eigen_compute)
    {					/* both "compute" and "set"	*/
        err_typ = ERR_ET1;		/* eigenvalues			*/
        ICh = 1;			/* sets to compute		*/
    }
    else if (eigen_compute)		
        ICh = 1;			/* sets to compute eigenvalues	*/

    else if (eigen_set)
        ICh = 0;			/* sets to eigenvalues were set	*/

    else
    {					/* sets to compute eigenvalues	*/
        err_typ = ERR_ET2;		/* (default)			*/
        ICh = 1;
    }
    
    if (!err_typ)
        return TRUE;
    else
        return FALSE; 
}

int test_peeling(void)			/* determines whether a peeling	*/
{					/* sequence file was named in   */
					/* command line or par statemnts*/

    pft = get_filent(PEEL);

    if (pft->command)			/* tests for peeling sequence   */
    {					/* file named in command line	*/
        if (pft->param)
            err_typ = ERR_PT1;
    }
    else if (pft->param)		/* tests for peeling sequence	*/
	     ;				/* file named in param statement*/

    if (!err_typ)
	return TRUE;
    else
	return FALSE;
}

