     
/*  functions are:
 *          allocPed()
 *          reallocRSDat()
 *          readPed()
 *          checkPed()
 *          openChkPed()
 *          writChkPed()
 *          assignChkPed()
 *          freeChkPed()
 */

#include "pedchk.h"

/*----------------------------------------------------------------------*/
/* allocPed():  allocates arrays for storing pedigree data              */

void allocPed(void)     
{
    pPedDat = malloc(PedSize * sizeof(PEDD));
    alloc_chk(WHERE, pPedDat);

    if (NNInts)
    {
        pIntDat = malloc(PedSize * (NNInts) * sizeof(int));
        alloc_chk(WHERE, pIntDat);
    }
    if (NReals && !DiscardReals)
    { 
        maxRSDat = FLTPT_CHR * NReals * PedSize;
        ppRSDat = malloc(PedSize * sizeof(char *));
        alloc_chk(WHERE, ppRSDat);
        *ppRSDat = malloc(maxRSDat * sizeof(char));
        alloc_chk(WHERE, *ppRSDat);
    }
    return;
}

/*----------------------------------------------------------------------*/
/* reallocRSDat():  increases size of array for storing reals           */
/*                  as character strings                                */

void reallocRSDat(void)
{
    char * pOld;
    int i;

    maxRSDat *= 1.2;
    pOld = *ppRSDat;
    *ppRSDat = realloc(*ppRSDat, maxRSDat * sizeof(char));
    alloc_chk(WHERE, *ppRSDat);
					/*this way to reassign pointers!*/
                                        /*see 7.29, C Programming FAQ bk*/
    for (i = 1; i <= nInd; ++i)
        *(ppRSDat + i) = *ppRSDat + (*(ppRSDat + i) - pOld);

    return;
}

/*----------------------------------------------------------------------*/
/* readPed():  reads pedigree data.  Checks data for valid format       */
/*             stores data in PedDat, IntDat and RSDat arrays        */

int nextInt(char **, char **, int *, int);
int nextReal(char **, char **, int);

int readPed(FILE * pF)
{
    int i, num, nchr;
    PED_PTR pP = pPedDat;
    PEDI_PTR pI = pIntDat;

    char pedRec[MAX_RECSIZ + 1];

    char *pStr, *pTok;
   					/*reads record for each individ	*/
 
    while (nInd < PedSize
                     && fgets(pedRec, MAX_RECSIZ + 1, pF) != NULL)
    {
        pStr = pedRec;                  /*gets individual's name,       */ 
					/*parents, gender (if present), */
        for (i = 0; i < NStdIn; ++i)    /*observed (if present)         */
        {
            if (!nextInt(&pStr, &pTok, &num, nInd + 1)) 
                return !OK; 
            if (num < 0)
            {
                give_err(ERC_fin, pTok, nInd + 1); 
                return !OK;
            }
            switch (StdIn[i]) 
            { 
                case NAM: 
                {
                    pP->nam = num;
                    if (num > maxPNam)
                        maxPNam = num; 
                    break; 
                }   
                case PA: 
                {
                    pP->namPa = num;
                    break;
                }   
                case MA:
                {
                    pP->namMa = num;
                    break;
                }   
                case SEX:
                {
                    pP->sex = num; 
                    if (pP->sex != MALE && pP->sex != FEMALE 
                                        && pP->sex != UNSEXED)
                    {
                        give_err(ERC_fsx, pTok, nInd + 1);
                        return !OK;
                    }
                    break;
                }   
                case OBS:
                {
                    pP->obsrvd = num;
                    if (pP->obsrvd != UNOBSERVED)
                        ++nObs;
                    break;
                }   
                default:
                    break;
            } 
            pStr = NULL;                /*required for strtok()         */
        }
                                        /*Tests for option to assign    */
                                        /*or ignore gender 	        */
        if (SexOp == ASSIGN) 
            pP->sex = UNSEXED;
                                        /*Tests for option to observe	*/
        if (ObsOp == ALL)               /*all individuals               */
            pP->obsrvd = OBSERVED;            

                                        /*gets the rest of the integers */ 
        for (i = 0; i < NNInts; ++i)
        {
            if (!nextInt(&pStr, &pTok, &num, nInd + 1))
                return !OK;

            *(pI + i) = num;
            if (num > maxPInt[i])
                maxPInt[i] = num;
            else if (num < minPInt[i])
                minPInt[i] = num;
         } 
                                        /*checks the reals              */
        if (NReals && !DiscardReals)    /*and stores the tokens         */
        {
            *(ppRSDat + nInd) = *ppRSDat + curRSDat;

            for (i = 0; i < NReals; ++i)  
            {
                if (!nextReal(&pStr, &pTok, nInd + 1))
                    return !OK;
                nchr = strlen(pTok);
    
                while (curRSDat + nchr + 3 >= maxRSDat)
                    reallocRSDat();

                strcpy(*ppRSDat + curRSDat, "  ");
                curRSDat += 2;
                strcpy(*ppRSDat + curRSDat, pTok);
                curRSDat += nchr;
            } 
            strcpy(*ppRSDat + curRSDat, "\0");
            ++curRSDat;
        }
        ++pP;
        pI += NNInts;
        ++nInd;
    }
    return OK;
}

int nextInt(char **ppS, char **ppT, int *n, int nR)
{					/*converts from character to	*/
    char *pEnd;	                 	/*intege                        */

    if (!(*ppT = strtok(*ppS, WHITESPACE)))
    {
        give_err(ERC_frm, nR);
        return !OK;
    }
    errno = 0;
    *n = strtol(*ppT, &pEnd, 10);        

    if (*pEnd != NUL)
    {
        give_err(ERC_fbi, *ppT, nR);
        return !OK;
    }

    if (errno == ERANGE)
    {
        give_err(ERC_fri, *ppT, nR);
        return !OK;
    }

    return OK;
}

int nextReal(char **ppS, char **ppT, int nR)
{					/*converts from character to	*/
    char *pEnd;	                 	/*double (must be valid 	*/
    double r;                           /*floating point number)        */

    if (!(*ppT = strtok(*ppS, WHITESPACE)))
    {
        give_err(ERC_frm, nR);
        return !OK;
    } 
    errno = 0;
    r = strtod(*ppT, &pEnd);        

    if (*pEnd != NUL)
    {
        give_err(ERC_fbr, *ppT, nR);
        return !OK;
    }
    if (errno == ERANGE)
    {
        give_err(ERC_frr, *ppT, nR); 
        return !OK;
    }

    return OK;
}

/*----------------------------------------------------------------------*/
/* checkPed():  checks pedigree size                                    */

int checkPed(FILE * pF)
{
    int r;

    fprintf(stdout, " %d individuals read\n\n", nInd);

    if (ObsOp == ALL)
        fprintf(stdout , " All individuals are observed\n\n");
    else if (ObsOp == USE)
        fprintf(stdout, " Number observed is %d\n\n", nObs);

    if (nInd < PedSize)
        give_warn(ERR_nex, PedSize);

    if (nInd < 3)
    {
        give_err(ERR_siz, "checkPed"); 
        return !OK;
    }
    else
    {
        r = fscanf(pF, "%*s");
        if (r != EOF)
            give_warn(ERR_urd, "checkPed");
    }
    return OK;
}

/*----------------------------------------------------------------------*/
/* openChkPed():  opens the output pedigree file                      */

FILE *openChkPed(void)
{
    int i, found;
    FILE * pOF;

    if (!get_filent(OPED)->fname)
    {
        give_err(ERC_nof);
        return !OK;
    }
    if (!(pOF = open_fil(OPED)))
        return NULL;

    fprintf(pOF, "input pedigree size %d\n", nInd);

    if (!NReals || DiscardReals)
        fprintf(pOF, "input pedigree record integers %d\n", 
                              NStdOut + NNInts);
    else
        fprintf(pOF, "input pedigree record integers %d reals %d\n",
                              NStdOut + NNInts, NReals);
    if (StdOut[1] == PA)
        fprintf(pOF, "input pedigree record father mother\n");
    else
        fprintf(pOF, "input pedigree record mother father\n");

    found = N;
    for (i = 0; i < NStdOut; ++i)
        if (StdOut[i] == SEX)
            found = Y;
    if (found == Y)
        fprintf(pOF, "input pedigree record gender present\n");
    else
        fprintf(pOF, "input pedigree record gender absent\n");
        
    found = N;
    for (i = 0; i < NStdOut; ++i)
        if (StdOut[i] == OBS)
            found = Y;
    if (found == Y) 
        fprintf(pOF, "input pedigree record observed present\n");
    else 
        fprintf(pOF, "input pedigree record observed absent\n");

    fprintf(pOF, "********\n");

    return pOF;
}

/*----------------------------------------------------------------------*/
/* writChkPed():  writes the pedigree file, in chronological order      */
/*                contains the same data as the input file, except      */
/*                reals may be discarded                                */

void writChkPed(FILE * pOF)
{
    KEYS_PP ppK = ppKeyTab;
    PED_PTR pP;
    PEDI_PTR pPI;
    
    int i, j, ixInd;

    widPNam = intWid(0, maxPNam);       /*gets field widths for format  */
    for (j = 0; j < NNInts; ++j)
        widPInt[j] = intWid(minPInt[j], maxPInt[j]);
           
					/*for each individual:		*/
    for (i = 0; i < nInd; ++i)
    {
        ixInd = (*ppK)->ix - 1;		/*gets index for next individual*/

        pP = pPedDat + ixInd;           /*outputs standard ints (name,  */
				        /*parents, gender, and if       */
                                        /*required, observed indicator) */	
        for (j = 0; j < NStdOut; ++j)
        {
            switch (StdOut[j]) 
            { 
                case NAM: 
                {
                    fprintf(pOF, "%*d", widPNam, pP->nam);
                    break; 
                }   
                case PA: 
                {
                    fprintf(pOF, " %*d", widPNam, pP->namPa);
                    break; 
                }   
                case MA:
                {
                    fprintf(pOF, " %*d", widPNam, pP->namMa);
                    break;
                }   
                case SEX:
                {
                    fprintf(pOF, "  %d", pP->sex);
                    break;
                } 
                case OBS:
                {
                    fprintf(pOF, "  %d", pP->obsrvd);
                    break;
                } 
                default:
                    break;
            }
                                    	/*outputs integers          	*/
        }
        fprintf(pOF, " ");

        pPI = pIntDat + NNInts * ixInd;	 

        for (j = 0; j < NNInts; ++j)
            fprintf(pOF, " %*d", widPInt[j], *(pPI + j)); 

                                    	/*outputs reals          	*/
        if (NReals && !DiscardReals)
            fprintf(pOF, "%s", *(ppRSDat + ixInd));

	fprintf(pOF, "\n");		/*ends current record		*/
        fflush(pOF);

        ++ppK;
    }
    return;
}

/*----------------------------------------------------------------------*/
/* assignChkPed():  sets the name of the input pedigree file to the     */
/*                  name of the output (check) file                     */
/*                  resets globals for input file to that of the        */
/*                  output file                                         */

int assignChkPed(void)
{
    int i;

    if (!(get_filent(PED)->fname = get_filent(OPED)->fname))
    {
        give_err(ERC_nof);
        return !OK;
    }

    NStdIn = NStdOut;
    NInts = NNInts + NStdOut;
    
    for (i = 0; i < NStdOut; ++i)
        StdIn[i] = StdOut[i];
    
    SexIn = SexOut;
    ObsIn = ObsOut;

    return OK;
}

/*----------------------------------------------------------------------*/
/* freeChkPed():  frees pedigree data arrays                            */

void freeChkPed(void)
{
    free(pPedDat);
    pPedDat = NULL;
    if (NNInts)
    {
        free(pIntDat);
        pIntDat = NULL;
    }
    if (NReals && !DiscardReals)
    {
        free(*ppRSDat);
        *ppRSDat = NULL;
        free(ppRSDat);
        ppRSDat = NULL;
    } 
    return;
}


