
#define PROGNAM "PolyEM bivar"
#define PNAM    "bivar"

#include "polyem.h"

extern double quarz(double *, double *, Term *, int);
extern double innerprod(double *, double *, int);

/*
 * program to test basic em equations : case of fixed + random + error 
 *
 * This May 1991 version uses the full G-inverse, instead of the G for 
 * observed individuals. It is adapted for a large pedigree, and takes as 
 * input the components of a sparse G-inverse, computed for all 
 * individuals, and also a set of eigenvalues of the G (hence G-inverse)
 * for observed individuals. 
 *
 * Evgen will now instead contain the eigenvalues of the INVERSE of the G 
 * for the observed individuals; 
 * biv_logdet, sigtrc, and tautrc have been modified accordingly
 *
 * modified (especially with regard to fixed effects) for greater 
 * flexibility and compatibility ; May 1992	
 *
 * modified, June 1996, by EAT, to allow reading in of predetermined 
 * eigenvalues as an option specified by the constant ICH 
 * modified, June 1996, by Chi-hong Tseng, to determine eigenvalues of 
 * A-matrix by components
 * modified, Aug 1996, by MAJ for more readable format for output data 
 * added function poly_print_pars 
 *
 * modified, Aug 1997, by MAJ, EAT and SCH for Version 2.0 revisions:   
 * use of parameter file for user-friendly specification of modelling 
 * parameters;
 * use of dynamic storage for all major arrays, with memory allocated 
 * just prior to use, and freed as soon as no longer needed;
 * addition of likelihood evaluation on looped pedigrees
 */

setinv(double *inv, double *sq)
{
    double cc = sq[0] * sq[2] - sq[1] * sq[1];
    inv[1] = -sq[1] / cc;
    inv[0] = sq[2] / cc;
    inv[2] = sq[0] / cc;
}

double sigtrc(double t1, double t2, double t3, double *evg)
{
    int j;
    double sv11, sv12, sv22, s11, s12, s22, cc = 0.0;

    for (j = 1; j <= np; ++j)
    {
	s11 = SigSq[0] / evg[j] + TauSq[0];
	s12 = SigSq[1] / evg[j] + TauSq[1];
	s22 = SigSq[2] / evg[j] + TauSq[2];
	sv11 = 1.0 / (s11 - s12 * s12 / s22);
	sv12 = -sv11 * s12 / s22;
	sv22 = 1.0 / (s22 - s12 * s12 / s11);
	cc += (t1 * sv11 + t2 * sv12 + t3 * sv22);
    }
    return (cc);
}

double tautrc(double t1, double t2, double t3, double t4, double *evg)
{
    int j;
    double sv11, sv12, sv22, s11, s12, s22, cc = 0.0;

    for (j = 1; j <= np; ++j)
    {
	s11 = SigSq[0] / evg[j] + TauSq[0];
	s12 = SigSq[1] / evg[j] + TauSq[1];
	s22 = SigSq[2] / evg[j] + TauSq[2];
	sv11 = 1.0 / (s11 - s12 * s12 / s22);
	sv12 = -sv11 * s12 / s22;
	sv22 = 1.0 / (s22 - s12 * s12 / s11);
	cc += (t1 - (t2 * sv11 + t3 * sv12 + t4 * sv22));
    }
    return (cc);
}

chk_sgfit(Term *ginv, int nt, double *evg, double *b1, double *b2)
{
    double temp, temp2, t1, t2, t3;

    t1 = TauSq[0] * SigSq[0];
    t3 = TauSq[1] * SigSq[1];
    t2 = TauSq[0] * SigSq[1] + TauSq[1] * SigSq[0];
    temp = sigtrc(t1, t2, t3, evg);
    temp2 = quarz(b1, b1, ginv, nt);
    SigSq[0] = (temp2 + temp) / np;
    printf(" check sig11;  trace =%11.3f,  quad =%11.3f,  net =%8.3f\n",
                                         	   temp, temp2, SigSq[0]);
    t1 = TauSq[0] * SigSq[1];
    t3 = TauSq[1] * SigSq[2];
    t2 = TauSq[1] * SigSq[1] + TauSq[0] * SigSq[2];
    temp = sigtrc(t1, t2, t3, evg);
    temp2 = quarz(b1, b2, ginv, nt);
    SigSq[1] = (temp2 + temp) / np;
    printf(" check sig12;  trace =%11.3f,  quad =%11.3f,  net =%8.3f\n",
	                                           temp, temp2, SigSq[1]);
    t1 = TauSq[1] * SigSq[1];
    t3 = TauSq[2] * SigSq[2];
    t2 = TauSq[2] * SigSq[1] + TauSq[1] * SigSq[2];
    temp = sigtrc(t1, t2, t3, evg);
    temp2 = quarz(b2, b2, ginv, nt);
    SigSq[2] = (temp2 + temp) / np;
    printf(" check sig22;  trace =%11.3f,  quad =%11.3f,  net =%8.3f\n\n",
	                                           temp, temp2, SigSq[2]);
}

chk_tufit(double *e1, double *e2, double *evg)
{					/* EM equations */
    double temp, temp2, t1, t2, t3, t4;

    t1 = TauSq[0];
    t2 = t1 * t1;
    t3 = 2.0 * t1 * TauSq[1];
    t4 = TauSq[1] * TauSq[1];
    temp = tautrc(t1, t2, t3, t4, evg);
    temp2 = innerprod(e1, e1, np);
    TauSq[0] = (temp2 + temp) / np;
    printf(" check tau11;  trace =%11.3f,  quad =%11.3f,  net =%8.3f\n",
	                                           temp, temp2, TauSq[0]);
    t1 = TauSq[1];
    t2 = TauSq[0] * t1;
    t3 = TauSq[0] * TauSq[2] + TauSq[1] * TauSq[1];
    t4 = TauSq[1] * TauSq[2];
    temp = tautrc(t1, t2, t3, t4, evg);
    temp2 = innerprod(e1, e2, np);
    TauSq[1] = (temp2 + temp) / np;
    printf(" check tau12;  trace =%11.3f,  quad =%11.3f,  net =%8.3f\n",
                                           	   temp, temp2, TauSq[1]);
    t1 = TauSq[2];
    t2 = TauSq[1] * TauSq[1];
    t3 = 2.0 * t1 * TauSq[1];
    t4 = t1 * t1;
    temp = tautrc(t1, t2, t3, t4, evg);
    temp2 = innerprod(e2, e2, np);
    TauSq[2] = (temp2 + temp) / np;
    printf(" check tau22;  trace =%11.3f,  quad =%11.3f,"
                                                    "  net =%8.3f\n\n\n",
	   temp, temp2, TauSq[2]);
}

sig_fit(Term * ginv, int nt, double *evg, double *b1, double *b2)

/* evg, b1, b2 - EM equations */
/* nt, ginv - terms of ginv */
{
    double temp, t1, t2, t3;

    t1 = TauSq[0] * SigSq[0];
    t3 = TauSq[1] * SigSq[1];
    t2 = TauSq[0] * SigSq[1] + TauSq[1] * SigSq[0];
    temp = sigtrc(t1, t2, t3, evg);
    SigSq[0] = (quarz(b1, b1, ginv, nt) + temp) / np;
    t1 = TauSq[0] * SigSq[1];
    t3 = TauSq[1] * SigSq[2];
    t2 = TauSq[1] * SigSq[1] + TauSq[0] * SigSq[2];
    temp = sigtrc(t1, t2, t3, evg);
    SigSq[1] = (quarz(b1, b2, ginv, nt) + temp) / np;
    t1 = TauSq[1] * SigSq[1];
    t3 = TauSq[2] * SigSq[2];
    t2 = TauSq[2] * SigSq[1] + TauSq[1] * SigSq[2];
    temp = sigtrc(t1, t2, t3, evg);
    SigSq[2] = (quarz(b2, b2, ginv, nt) + temp) / np;
}

tau_fit(double *e1, double *e2, double *evg)
{							/* EM equations */
    double temp, t1, t2, t3, t4;

    t1 = TauSq[0];
    t2 = t1 * t1;
    t3 = 2.0 * t1 * TauSq[1];
    t4 = TauSq[1] * TauSq[1];
    temp = tautrc(t1, t2, t3, t4, evg);
    TauSq[0] = (innerprod(e1, e1, np) + temp) / np;
    t1 = TauSq[1];
    t2 = TauSq[0] * t1;
    t3 = TauSq[0] * TauSq[2] + TauSq[1] * TauSq[1];
    t4 = TauSq[1] * TauSq[2];
    temp = tautrc(t1, t2, t3, t4, evg);
    TauSq[1] = (innerprod(e1, e2, np) + temp) / np;
    t1 = TauSq[2];
    t2 = TauSq[1] * TauSq[1];
    t3 = 2.0 * t1 * TauSq[1];
    t4 = t1 * t1;
    temp = tautrc(t1, t2, t3, t4, evg);
    TauSq[2] = (innerprod(e2, e2, np) + temp) / np;
}

double biv_logdet(double ev[])
{
    int j;
    double r1, r2, s11, s12, s22, toto, old, cc = 0.0;

    old = 0.0;
    for (j = 1; j <= np; ++j)
    {
	if (ev[j] != old)
	{
	    s11 = SigSq[0] / ev[j] + TauSq[0];
	    s12 = SigSq[1] / ev[j] + TauSq[1];
	    s22 = SigSq[2] / ev[j] + TauSq[2];
	    toto = (s11 - s22) * (s11 - s22) + 4.0 * s12 * s12;
	    toto = sqrt(toto);
	    if (s12)
	    {
		r1 = ((s11 - s22) + toto);
		r2 = ((s11 - s22) - toto);
	    } else
		r1 = r2 = 0.0;
	    old = ev[j];
	}
	cc += log(.5 * r1 + s22);
	cc += log(.5 * r2 + s22);
    }
    return (cc);
}

/* Note within this routine "xx" are the residuals from any fixed effects;
 * i.e. "y" and NOT original data x 
 *
 * Re-written to allow for looped pedigrees; June 1997 (SCH) 
 *
 * IMPORTANT: Assumes that ginv contains the *lower left* non-zero terms 
 * of the full G-inverse.  The henderson() routine has been modified to do
 * this (originally it produced the *upper right* terms).  This, of course,
 * only affects the order in which the terms are stored. Having the lower
 * left terms means that itermo() can efficiently work from the bottom of 
 * the pedigree upwards, which improves initial convergence if data is 
 * mainly present at the bottom of the pedigree. 
 */

void itermo(double *xx1, double *xx2, double *b1, double *b2, 
                                                     Term * ginv, int nt)
{
    int i, j, it, k, k1, n;
    double cc[3], cinv[3], z, d[2], zz[2];
    double siginv[3], tauinv[3], *t1[2];

    t1[0] = malloc(sizeof(double) * pedsize * 2);
    alloc_chk(WHERE, t1);
    t1[1] = t1[0] + pedsize;
    setinv(siginv, SigSq);
    setinv(tauinv, TauSq);
    for (it = 0; it < ItMo; it++)
    {
	for (i = 0; i < pedsize; i++)
	{
	    if (j = qdex[i])
	    {
		t1[0][i] = xx1[j] * tauinv[0] + xx2[j] * tauinv[1];
		t1[1][i] = xx1[j] * tauinv[1] + xx2[j] * tauinv[2];
	    } else
		t1[0][i] = t1[1][i] = 0.0;
	}
	d[0] = d[1] = 0.0;
	n = 0;
	k = nt;
	do
	{
	    i = ginv[k].r1;
	    z = ginv[k--].value;
	    for (j = 0; j < 3; j++)
		cc[j] = siginv[j] * z;
	    if (qdex[i])
		for (j = 0; j < 3; j++)
		    cc[j] += tauinv[j];
	    setinv(cinv, cc);
	    k1 = k;
	    while (k > 0 && ginv[k].r1 == i)
	    {
		j = ginv[k].r2;
		z = ginv[k--].value;
		t1[0][i] -= z * (b1[j] * siginv[0] + b2[j] * siginv[1]);
		t1[1][i] -= z * (b1[j] * siginv[1] + b2[j] * siginv[2]);
	    }
	    zz[0] = (t1[0][i] * cinv[0] + t1[1][i] * cinv[1]);
	    zz[1] = (t1[0][i] * cinv[1] + t1[1][i] * cinv[2]);
	    d[0] += (b1[i] - zz[0]) * (b1[i] - zz[0]);
	    d[1] += (b2[i] - zz[1]) * (b2[i] - zz[1]);
	    n++;
	    b1[i] = zz[0];
	    b2[i] = zz[1];
	    k = k1;
	    zz[0] = b1[i] * siginv[0] + b2[i] * siginv[1];
	    zz[1] = b1[i] * siginv[1] + b2[i] * siginv[2];
	    while (k > 0 && ginv[k].r1 == i)
	    {
		j = ginv[k].r2;
		z = ginv[k--].value;
		t1[0][j] -= z * zz[0];
		t1[1][j] -= z * zz[1];
	    }
	}
	while (k > 0);
	z = d[0] / SigSq[0] + d[1] / SigSq[2];

	if (0.5 * z / (double) n < Diff)
	    break;
    }
    free(t1[0]);
    t1[0] = NULL;
}

breeding_values(double *z1, double *z2, double *a1, double *a2, 
                double *b1, double *b2, double *e1, double *e2, 
                                                    Term * ginv, int nt)
{
    int i;

    itermo(z1, z2, b1, b2, ginv, nt);

    for (i = 0; i < pedsize; ++i)
    {
	a1[qdex[i]] = b1[i];
	a2[qdex[i]] = b2[i];
    }
    for (i = 1; i <= np; ++i)
    {
	e1[i] = z1[i] - a1[i];
	e2[i] = z2[i] - a2[i];
    }
}

biv_likval(int k, double *evgen, double *z1, double *z2, double *a1, 
                  double *a2, double *b1, double *b2, Term * ginv, int nt)
{
    double lhood;
    double tauinv[3], siginv[3];

    lhood = 2.0 * np * log(PPI);
    lhood += biv_logdet(evgen);
    setinv(tauinv, TauSq);
    lhood += tauinv[0] * (innerprod(z1, z1, np) - innerprod(a1, a1, np));
    lhood += 2.0 * tauinv[1] * 
                         (innerprod(z1, z2, np) - innerprod(a1, a2, np));
    lhood += tauinv[2] * (innerprod(z2, z2, np) - innerprod(a2, a2, np));

    setinv(siginv, SigSq);
    lhood -= siginv[0] * quarz(b1, b1, ginv, nt);
    lhood -= 2.0 * siginv[1] * quarz(b1, b2, ginv, nt);
    lhood -= siginv[2] * quarz(b2, b2, ginv, nt);

    lhood = -0.5 * lhood;
    poly_print_pars(k, 4);
    printf("\n current log-likelihood =");
    print_val(lhood);
    printf("\n\n\n");
    fflush(stdout);
}

em_iter(int k, double *a1, double *a2, double *b1, double *b2, 
        double *epsi1, double *epsi2, double *z1, double *z2, 
        Term * ginv, int nt, double *evgen)
{
    				/* EM iteration	*/
    				/* iterate for expected breeding values */
    int i;
    double temp;

    out_fixed(z1, 0, 0);
    out_fixed(z2, 1, 1);
    breeding_values(z1, z2, a1, a2, b1, b2, epsi1, epsi2, ginv, nt);

    /* now evaluate likelihood */

    if ((k / ItWt) * ItWt == k)
    {
	biv_likval(k, evgen, z1, z2, a1, a2, b1, b2, ginv, nt);
        if (ChkTrc)
	{   chk_sgfit(ginv, nt, evgen, b1, b2);
	    chk_tufit(epsi1, epsi2, evgen); 	 
        }
    }				/* now implement EM equations for new */
				/* variances  */
    sig_fit(ginv, nt, evgen, b1, b2);
    tau_fit(epsi1, epsi2, evgen);

    /* now backfit for each class of fixed effects */
    if (NFix == 0)
    {
	out_fixed(z1, 0, 0);
	out_fixed(z2, 1, 1);
 	breeding_values(z1, z2, a1, a2, b1, b2, epsi1, epsi2, ginv, nt);
	mu[0] = 0.0;
	mu[1] = 0.0;
	for (i = 1; i <= np; ++i)
	{
	    z1[i] = (x[0][i] - a1[i]);
	    mu[0] += z1[i];
	}
	for (i = 1; i <= np; ++i)
	{
	    z2[i] = (x[1][i] - a2[i]);
	    mu[1] += z2[i];
	}
	mu[0] = mu[0] / np;
	mu[1] = mu[1] / np;
	Mufti[0][0][1] = mu[0];
	Mufti[1][0][1] = mu[1];
	tau_fit(epsi1, epsi2, evgen);
    } else
    {
	for (i = 1; i <= NFix; ++i)
	{
	    out_fixed(z1, 0, 0);
	    out_fixed(z2, 1, 1);
 	    breeding_values(z1, z2, a1, a2, b1, b2, epsi1, epsi2, 
                                                               ginv, nt);
	    out_bar_one(a1, z1, x[0], i, 0);
	    out_bar_one(a2, z2, x[1], i, 1);
	    temp = fixeffit(z1, i, 0);
	    mu[0] = temp;
	    Mufti[0][0][1] = temp;
	    temp = fixeffit(z2, i, 1);
	    mu[1] = temp;
	    Mufti[1][0][1] = temp;
	    tau_fit(epsi1, epsi2, evgen);
	}
    }
}

setvals(double *b1, double *b2)
{
    int i;

    mu[0] = Mufti[0][0][1];
    mu[1] = Mufti[1][0][1];

    /* set initial breeding values */
    for (i = 0; i < pedsize; ++i)
    {
	if (!qdex[i])
	    b1[i] = b2[i] = 0.0;
	else
	{
	    b1[i] = (SigSq[0] / (TauSq[0] + SigSq[0])) 
                                         * (x[0][qdex[i]] - mu[0]);
	    b2[i] = (SigSq[2] / (TauSq[2] + SigSq[2])) 
                                         * (x[1][qdex[i]] - mu[1]);
	}
    }
    x[0][0] = 0.0;
    x[1][0] = 0.0;
}

alloc_vect(double **pa1, double **pepsi1, double **pz1, double **pb1,
	   double **pa2, double **pepsi2, double **pz2, double **pb2)
{
    *pa1 = malloc((np + 1) * sizeof(double));
    alloc_chk(WHERE, *pa1);
    *pepsi1 = malloc((np + 1) * sizeof(double));
    alloc_chk(WHERE, *pepsi1);
    *pz1 = malloc((np + 1) * sizeof(double));
    alloc_chk(WHERE, *pz1);
    *pb1 = malloc(pedsize * sizeof(double));
    alloc_chk(WHERE, *pb1);
    *pa2 = malloc((np + 1) * sizeof(double));
    alloc_chk(WHERE, *pa2);
    *pepsi2 = malloc((np + 1) * sizeof(double));
    alloc_chk(WHERE, *pepsi2);
    *pz2 = malloc((np + 1) * sizeof(double));
    alloc_chk(WHERE, *pz2);
    *pb2 = malloc(pedsize * sizeof(double));
    alloc_chk(WHERE, *pb2);
}

free_vect(double *a1, double *epsi1, double *z1, double *b1,
	  double *a2, double *epsi2, double *z2, double *b2,
	                             double *evgen, Term * ginverse)
{
    free((void *) a1);
    a1 = NULL;
    free((void *) epsi1);
    epsi1 = NULL;
    free((void *) z1);
    z1 = NULL;
    free((void *) b1);
    b1 = NULL;
    free((void *) a2);
    a2 = NULL;
    free((void *) epsi2);
    epsi2 = NULL;
    free((void *) z2);
    z2 = NULL;
    free((void *) b2);
    b2 = NULL;

    free((void *) evgen);
    evgen = NULL;  
    free((void *) ginverse);
    ginverse = NULL;
}

int main(int argc, char *argv[])
{
    int i, j, k, cpnp, *cdex;
    int **dodo = NULL;
    double *a1, *epsi1, *z1, *b1, *a2, *epsi2, *z2, *b2;
    double *evgen, *evgen1;
    int gsize;
    Ind **cpnghd;
    Term *ginverse;
    Indlist *peopl;


    atexit(print_tail);
    print_head(PROGNAM);

    /* process all of the input parameter statements */

    if (!proc_pars(argc, argv, PNAM))
	exit(EXIT_FAILURE);

    if (!proc_poly())
	exit(EXIT_FAILURE);

    if (!proc_fils())
	exit(EXIT_FAILURE);

    /* reads and checks the pedigree file for pedigree errors */

    if (!chk_pedfil())
        give_up(ERR_qit);	

    /* if the pedigree file was not in chronological order, */
    /* writes a new one */

    if (!mak_opedfil())
        give_up(ERR_qit);	

    /* reopen the pedigree file for reading the data */

    if (!(pfile = open_ped()))
	give_up(ERR_qit);	

    alloc_ped();

    /* this routine now also calls nghds() and sortdat() */

    input_data(pfile, 0, dodo);
    fclose(pfile);

    free_ped();

    /* the no. of observed individuals is known; check against limit */
    /* also the no. of fixed effects levels is known; check against */
    /* no. starting values */

    if (!check_pars(np))
        exit(EXIT_FAILURE);

    /* evgen will now instead contain the eigenvalues of the INVERSE */
    /* of G-obs; */
    /* logdet, sigtrc, and tautrc have been modified accordingly */

    ginverse = malloc(pedsize * 4 * sizeof(Term));
    alloc_chk(WHERE, ginverse);

    /* read or find non-zero terms of g-inverse for all pedigree */
    /* individuals */

    gsize = henderson(ginverse);
    ginverse = realloc(ginverse, (gsize + 1) * sizeof(Term));
    alloc_chk(WHERE, ginverse);

    /* For testing; print G-inverse, and if eigenvalues are not to be */
    /* printed, exit */

    if (ChkGnv)
    {   printf(" G-inverse:\n\n");
        for (i = 1; i <= gsize; ++i)
	{
            printf(" %5d %5d %5d", i, ginverse[i].r1, ginverse[i].r2);
            print_val(ginverse[i].value);
            printf("\n");
        }
        if (!ChkEig)
            exit(EXIT_SUCCESS);
        else
	    printf("\n");
    }

    /* find or read eigenvalues of G-inverse for OBSERVED individuals  */

    /* cdex : component index     cpnghd : component nghd */

    if (ICh == 1)
    {
	cdex = malloc(pedsize * sizeof(int));
        alloc_chk(WHERE, cdex);
	cpnghd = malloc(max_cpsize * sizeof(Ind *));
        alloc_chk(WHERE, cpnghd);
	evgen1 = malloc((max_cpsize + 1) * sizeof(double));
        alloc_chk(WHERE, evgen1);
	evgen = malloc((np + 1) * sizeof(double));
        alloc_chk(WHERE, evgen);

	j = 1;
	for (k = 1; k < n_compnts + 1; k++)
	{
	    i = 0;
	    cpnp = 0;			/* no. NOT observed in component*/
	    peopl = cmplist[k].people;
	    while (peopl->next != NULL)
	    {
		peopl = peopl->next;
		cpnghd[i] = peopl->current;
		if (qdex[cpnghd[i]->index] == 0)
		    cpnp++;
		cdex[cpnghd[i]->index] = i;
		i++;
	    }
            if (i > cpnp)
         	bigcheig(evgen1, i - cpnp, i, cpnghd, cdex);
            else if (i == cpnp)
                give_warn(ERR_noo, i, cpnghd[0]->self);

	    while (i > cpnp)
	    {
		evgen[j] = evgen1[i - cpnp];
		i--;
		j++;
	    }
	}
	free((void *) evgen1);
	evgen1 = NULL;
	free((void *) cdex);
	cdex = NULL;
	free((void *) cpnghd);
	cpnghd = NULL;
        printf(" Eigenvalues have been computed\n\n\n");
    } 
    else if (ICh == 0)
    {   
        evgen = peigen;
        printf(" Eigenvalues have been read from parameter file\n\n\n");
    }
    else if (ICh == -1)
    {
        if (!input_eigen(np))
            exit(EXIT_FAILURE);
        else 
        {
            evgen = peigen;
            printf(" Eigenvalues have been read from eigenvalues" 
                                                       " file\n\n\n"); 
        }
    }
    free_cmplists();
    free_nghds();

    /* For testing bigcheig and reduce routines;  check terms and exit */

    if (ChkEig)
    {   printf(" eigenvalues of G-observed:\n\n");
        for (i = 1; i <= np; ++i)
        {
            print_val5(evgen[i]);
            if (i%10 == 0)
                printf("\n");  
        }
	printf("\n\n");
        exit(EXIT_SUCCESS);
    }
    for (i = 1; i <= np; ++i)
	evgen[i] = 1.0 / evgen[i];

    /* allocate space for vectors  */

    alloc_vect(&a1, &epsi1, &z1, &b1, &a2, &epsi2, &z2, &b2);

    z1[0] = 0.0;
    z2[0] = 0.0;

    /* set initial values  */

    setvals(b1, b2);

    /* now ready for EM iteration  */

    for (k = 0; k <= ItEM; ++k)
	em_iter(k, a1, a2, b1, b2, epsi1, epsi2, z1, z2, ginverse,
                                                          gsize, evgen);

    out_fixed(z1, 0, 0);
    out_fixed(z2, 1, 1);
    breeding_values(z1, z2, a1, a2, b1, b2, epsi1, epsi2, ginverse, gsize);
    biv_likval(k, evgen, z1, z2, a1, a2, b1, b2, ginverse, gsize);

    free_input_data();
    free_vect(a1, epsi1, z1, b1, a2, epsi2, z2, b2, evgen, ginverse);

    return 0;
}

